/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.codegen;

// Kelp imports
import org.enhydra.kelp.common.Constants;
//
import java.io.PrintStream;

/**
 * The WebAppException is thrown for exceptions in the
 * WebAppUtil class.
 */
public class CodeGenException extends Exception {
    private Throwable parent = null;
    private boolean   decoy = false;

    /**
     * Create a GeneratorException with a specified error message.
     *
     * @param message
     * Message providing information on the nature of the exception.
     */
    public CodeGenException(String message) {
        super(message);
    }

    /**
     * Constructor declaration
     *
     *
     * @param hidden
     *
     * @see
     */
    public CodeGenException(Throwable hidden) {
        super(new String());
        parent = hidden.fillInStackTrace();
        decoy = true;
    }

    /**
     * Constructor declaration
     *
     *
     * @param chain
     * @param message
     *
     * @see
     */
    public CodeGenException(Throwable chain, String message) {
        super(message);
        parent = chain.fillInStackTrace();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getMessage() {
        StringBuffer buf = new StringBuffer();

        if ((!decoy) || (parent == null)) {
            buf.append(super.getMessage());
        }
        if (parent != null) {
            buf.append('\n');
            buf.append(Constants.TAB4);
            buf.append(parent.getMessage());
        }
        return buf.toString();
    }

    public void printStackTrace() {
        super.printStackTrace();
        if (parent != null) {
            parent.printStackTrace();
        }
    }

    public void printStackTrace(PrintStream stream) {
        super.printStackTrace(stream);
        if (parent != null) {
            parent.printStackTrace(stream);
        }
    }



}
