/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

package org.enhydra.kelp.common.dods;

// Kelp imports
import org.enhydra.kelp.common.AbstractTool;

// ToolBox imports
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.SwingUtil;

// Standard imports
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

//Dusan
import java.io.File;
import org.enhydra.dods.generator.DODSGenerator;

//
public class CoreDodsTool extends AbstractTool {
    static ResourceBundle addinRes =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    private DodsInnerPanel  innerPanel = null;
    private DodsButtonPanel buttonPanel = null;
    DODSGenerator generator;

    public static final String getDefaultTitle() {
      return "DODS Generator";
    }

    public static void main(String[] args) {
        //Dusan 16.3.2003.
        System.setProperty("DODS_HOME", "C:"+File.separator+"enhydra5.1"+
          File.separator+"dods");

        SwingUtil.setLookAndFeelToSystem();
        CoreDodsTool tool = null;

        tool = new CoreDodsTool();
        tool.showDialog(new Frame());
        System.exit(0);
    }

    public CoreDodsTool() {
        super();
    }

    // override AbstractTool
    public void clearAll() {
        super.clearAll();
        innerPanel = null;
        buttonPanel = null;
    }

    // implement AbstractTool
    public String getTitle() {
      return CoreDodsTool.getDefaultTitle();
    }

    // implement AbstractTool
    public ActionListener createButtonListener() {
        return (new DodsButtonListener(this));
    }

    // implement AbstractTool
    public InnerPanel getInnerPanel() {
        if (innerPanel == null) {
            innerPanel = new DodsInnerPanel();
        }
        return innerPanel;
    }

    // implement AbstractTool
    public ButtonPanel getButtonPanel() {
        if (buttonPanel == null) {
            buttonPanel = new DodsButtonPanel();
        }
        return buttonPanel;
    }

    // implement AbstractTool
    protected String getProgressTitle() {
        return addinRes.getString("Dods_Progress");
    }

    protected void back() {
      innerPanel.back();
    }

    protected void generate() {
       innerPanel.generate(this);
    }

}
