/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte;

import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.PropUtil;

import org.enhydra.kelp.forte.node.ForteProject;

import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import java.io.*;
import java.util.Hashtable;

/** Options for something or other.
 *
 * @author  rees0234
 */
public class XMLCSettings extends SystemOption implements PropertyKeys
{
    static final long serialVersionUID = -1412625156769505620L;
    
    protected Hashtable nodeProps = null;
        
   public void writeExternal (ObjectOutput out) throws IOException {
        out.writeObject(propertyValue(XMLC_DOC_TYPES));
        out.writeObject(propertyValue(XMLC_BUILD));
        out.writeObject(propertyValue(XMLC_PRINT_DOM));
        out.writeObject(propertyValue(XMLC_PRINT_PARSE));
        out.writeObject(propertyValue(XMLC_PRINT_DOC));
        out.writeObject(propertyValue(XMLC_VERBOSE));
        out.writeObject(propertyValue(XMLC_PRINT_ACCESSOR));
        out.writeObject(propertyValue(XMLC_OUTPUT_FILENAME));
        out.writeObject(propertyValue(XMLC_MAP_PACKAGE_FROM));
        out.writeObject(propertyValue(XMLC_MAP_PACKAGE_TO));
        out.writeObject(propertyValue(XMLC_MAP_PACKAGE_LENGTH));
        out.writeObject(propertyValue(XMLC_MAP_SCOPE));
        out.writeObject(propertyValue(NAME_XMLC_OPTION_FILEPATH));
        out.writeObject(propertyValue(NAME_XMLC_PARAMETERS));
        out.writeObject(propertyValue(PROP_PROJECT_ROOT));
        out.writeObject(propertyValue(PROP_SRC_DIR));
        out.writeObject(nodeProps);
    }

   public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        setProperty(XMLC_DOC_TYPES, (String)in.readObject());
        setProperty(XMLC_BUILD, (String)in.readObject());
        setProperty(XMLC_PRINT_DOM, (String)in.readObject());
        setProperty(XMLC_PRINT_PARSE, (String)in.readObject());
        setProperty(XMLC_PRINT_DOC, (String)in.readObject());
        setProperty(XMLC_VERBOSE, (String)in.readObject());
        setProperty(XMLC_PRINT_ACCESSOR, (String)in.readObject());
        setProperty(XMLC_OUTPUT_FILENAME, (String)in.readObject());
        setProperty(XMLC_MAP_PACKAGE_FROM, (String)in.readObject());
        setProperty(XMLC_MAP_PACKAGE_TO, (String)in.readObject());
        setProperty(XMLC_MAP_PACKAGE_LENGTH, (String)in.readObject());        
        setProperty(XMLC_MAP_SCOPE, (String)in.readObject());
        setProperty(NAME_XMLC_OPTION_FILEPATH, (String)in.readObject());
        setProperty(NAME_XMLC_PARAMETERS, (String)in.readObject());
        setProperty(PROP_PROJECT_ROOT, (String)in.readObject());
        setProperty(PROP_SRC_DIR, (String)in.readObject());
 
        try {
            nodeProps = (Hashtable)in.readObject();
        }  catch (OptionalDataException e) {
            if (nodeProps == null)
                nodeProps = new Hashtable();
        }
   }


    public String displayName () {
        return NbBundle.getMessage (XMLCSettings.class, "LBL_xmlc_settings");
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you provide context help then use:
        // return new HelpCtx (XMLCSettings.class);
    }

    /** Default instance of this system option, for the convenience of associated classes. */
    public static XMLCSettings getDefault ()
    {
        return (XMLCSettings) findObject (XMLCSettings.class, true);
    }

    public void setProperty(String key, String value)
    {
        putProperty(key, value, true);
    }

    public String propertyValue(String key)
    {
        return (String)getProperty(key);
    }

    public void setNodeProperty(String key, String prop)
    {
        if (nodeProps == null)
            nodeProps = new Hashtable();

        if (prop != null)
            try {
                nodeProps.put(key, prop);
            }catch (NullPointerException e) {
                System.err.println(e);
            }
        else
            nodeProps.remove(key);
    }

    public String getNodeProperty(String key)
    {
        if (nodeProps != null)
        {
            String res = (String)nodeProps.get(key);
            return res;
        }

        return null;
    }
    
    public boolean isGlobal(){
        return false;
    }
//this is going to be absolute
    public String getRoot()
    {
        ForteProject proj = new ForteProject();
        String cur = propertyValue(PROP_PROJECT_ROOT);
        if ((cur == null) || (cur.length() <= 0))
        {
            cur = proj.getDefaultRoot();
            if (cur != null)
                setRoot(cur);
        }
        if (! proj.inProject(cur))
        {
            cur = proj.getDefaultRoot();
            if (cur != null)
                setRoot(cur);
        }
            
        return cur;        
    }
    
    public void setRoot(String root)
    {
        setProperty(PROP_PROJECT_ROOT, root);
    }
//this is relative to root    
    public String getSource()
    {
        String res = propertyValue(PROP_SRC_DIR);
        if ((res == null) || (res.length() <= 0))
        {
            res = "src";
            setSource(res);
        }
        return res;
    }
    
    public void setSource(String src)
    {
        setProperty(PROP_SRC_DIR, src);
    }
    
/*    public boolean isBuild () {

       return readBooleanProperty(XMLC_BUILD);

    }

    public void setBuild (boolean build) {
        writeBooleanProperty(XMLC_BUILD, build);
    }
    
       public void writeBooleanProperty(String property, boolean b) {
        String flag = b ? Boolean.TRUE.toString() : Boolean.FALSE.toString();
        setProperty(property, flag);
    } */

}
