/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */


package org.enhydra.kelp.forte.actions;

import java.awt.event.ActionEvent;
import java.awt.Component;
import javax.swing.*;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.Presenter;
import org.openide.util.actions.SystemAction;
/**
 *
 * @author  rees0234
 * @version
 */
public class EnhydraAction extends SystemAction implements Presenter.Menu, Presenter.Popup {

    public void actionPerformed (ActionEvent ev) {
        // do nothing; should not be called
    }

    public String getName () {
        return NbBundle.getMessage (EnhydraAction.class, "LBL_Enhydra_Tools");
    }

    protected String iconResource () {
        return "XMLCActionIcon.gif";
    }

    public HelpCtx getHelpCtx () {
   //     return new HelpCtx(EnhydraAction.class);
         return HelpCtx.DEFAULT_HELP;
    }

    /** List of system actions to be displayed within this one's toolbar or submenu. */
    private static final SystemAction[] grouped = new SystemAction[] {
                SystemAction.get (XMLCAction.class),
                SystemAction.get (DeploymentAction.class),
                SystemAction.get (AppWizardAction.class),
            };

    private static Icon icon = null;

    public JMenuItem getMenuPresenter () {
        JMenu menu = new JMenu (getName ());
        if (icon == null) icon = new ImageIcon (EnhydraAction.class.getResource (iconResource ()));
        menu.setIcon (icon);
        for (int i = 0; i < grouped.length; i++) {
            SystemAction action = grouped[i];
            if (action == null) {
                menu.addSeparator ();
            } else if (action instanceof Presenter.Menu) {
                menu.add (((Presenter.Menu) action).getMenuPresenter ());
            }
        }
        return menu;
    }

    public JMenuItem getPopupPresenter () {
        JMenu menu = new JMenu (getName ());
        // Conventional not to set an icon here.
        for (int i = 0; i < grouped.length; i++) {
            SystemAction action = grouped[i];
            if (action == null) {
                menu.addSeparator ();
            } else if (action instanceof Presenter.Popup) {
                menu.add (((Presenter.Popup) action).getPopupPresenter ());
            }
        }
        return menu;
    }

}
