/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */
package org.enhydra.kelp.forte.actions;

import org.openide.nodes.Node;
import org.openide.TopManager;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.actions.SystemAction;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.src.ClassElement;
import org.openide.DialogDescriptor;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;

// ToolBox
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.common.event.HelpListener;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.xmlc.XMLCButtonListener;
import org.enhydra.kelp.common.xmlc.XMLCButtonPanel;
import org.enhydra.kelp.common.xmlc.CoreXMLCTool;
import org.enhydra.kelp.common.node.OtterDocumentNode;

// AddinForte
import org.enhydra.kelp.forte.node.ForteProject;
import org.enhydra.kelp.forte.XMLCSettings;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.util.Enumeration;

//dr
import org.enhydra.kelp.common.node.OtterProject;

/**
 * Action sensitive to some cookie that does something useful.
 *
 * @author  rees0234
 */
public class XMLCAction extends CallableSystemAction implements HelpListener {
    private static String anchor = "xmlcwizard";

    // implements HelpListener
    public void onHelp(HelpEvent event) {
        URL          url = null;
        String[]     home = new String[2];
        home[0] = System.getProperty("netbeans.home");
        home[1] = System.getProperty("netbeans.user");
        StringBuffer buf = new StringBuffer();
        buf.append(KelpInfo.getAddinHelpURL(home));
        buf.append('#').append(anchor);
        try {
          url = new URL(buf.toString());
          TopManager.getDefault().showUrl(url);
        } catch (MalformedURLException e) {
          e.printStackTrace(System.err);
        }
    }


    public void performAction() {
        String[]           options = XMLCButtonPanel.getOptions();
        CoreXMLCTool       tool = new CoreXMLCTool();
        XMLCButtonListener listener = new XMLCButtonListener(tool);
        DialogDescriptor   dscr = new DialogDescriptor(tool.getInnerPanel(),
                tool.getTitle(), true, options, options[0],
                DialogDescriptor.BOTTOM_ALIGN, HelpCtx.DEFAULT_HELP,
                listener);
        Dialog             dlg = TopManager.getDefault().createDialog(dscr);
        tool.setProject(new ForteProject());
        tool.addHelpListener(this);
        dlg.show();

        //gotta refresh the directories so build will work right away on generated files:
        ForteProject project = new ForteProject();
        String src = XMLCSettings.getDefault().getSource();
        if ((src != null) && (src.length() > 0))
        {
            FileObject source = project.getRootFolder().getFileObject(src);
            source.refresh();
            Enumeration kids = source.getChildren(true);
            while (kids.hasMoreElements())
                ((FileObject)kids.nextElement()).refresh();  //blech
        }


    }

    public boolean isEnabled()
    {
        ForteProject project = new ForteProject();
        if (project.getRootFolder() == null)
//            firePropertyChange(PROP_ENABLED, (Boolean)getProperty(PROP_ENABLED), new Boolean(false);
            return false;
        OtterDocumentNode[] nodes = project.getAllDocuments();
        return ((nodes != null) && (nodes.length > 0));
    }

    public String getName() {
        return CoreXMLCTool.getDefaultTitle();
    }

    protected String iconResource() {
        return "smallicon.gif";
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (XMLCAction.class);
    }

    /**
     * Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
     */
    protected void initialize() {
        // mn 08.06.2002
        super.initialize();
        putProperty(javax.swing.Action.SHORT_DESCRIPTION,
                    NbBundle.getMessage(XMLCAction.class,
                                        "HINT_XMLC_Action"));
    }

}
