/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.PropertyKeys;

// NetBeans imports
import org.openide.nodes.Node;

// Standard imports
import java.io.IOException;

//
/**
 * Class declaration
 *
 *
 * @author
 */
public class ForteDocumentNode extends ForteNode implements OtterDocumentNode {

 //   XMLCDataNode nativeNode;
 //dr
// XMLCDataNode nativeNode;
    //
    public ForteDocumentNode(XMLCDataObject n) {
        super(n);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterNode
     * @param path
     */
    public ForteDocumentNode(OtterNode otterNode, String path) {
        super(otterNode, path);
    }

    public boolean isSelected()
    {
        String in = getProperty(NAME_SELECTED);
        return PropUtil.stringToBoolean(in, false);
    }

    public void setSelected(boolean b) {
        setProperty(NAME_SELECTED, PropUtil.booleanToString(b));
        if (b) {
            setStatic(false);
        }
    }

    public boolean isStatic() {
        String in = getProperty(NAME_RESOURCE_STATIC);

        return PropUtil.stringToBoolean(in, false);
    }

    public void setStatic(boolean b) {
        setProperty(NAME_RESOURCE_STATIC,
                    PropUtil.booleanToString(b));
        if (b) {
            setSelected(false);
        }
    }

    public void preCompile() {

/*       if (getNativeNode() instanceof FileNode) {
            XMLCDataNode nativeNode = (XMLCDataNode) getNativeNode();

            if (nativeNode.isModified()) {
                try {
                    nativeNode.save();
                } catch (InvalidUrlException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } */
    }

    public String getProperty(String property)
    {
        String out = "";

        if (nativeNode != null) {
                out = ((XMLCDataObject)nativeNode).getProperty(property);
        }

        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value)
    {
        if (nativeNode != null) {
                ((XMLCDataObject)nativeNode).setProperty(property, value);
        }
    }

    public String getXMLCOptionFilePath()
    {
        return getProperty(NAME_XMLC_OPTION_FILEPATH);
    }

    public void setXMLCOptionFilePath(String in)
    {
        setProperty(NAME_XMLC_OPTION_FILEPATH, in);    }

    public String getXMLCParameters()
    {
        return getProperty(NAME_XMLC_PARAMETERS);
    }

    public void setXMLCParameters(String in)
    {
        setProperty(NAME_XMLC_PARAMETERS, in);
    }
}
