/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.services;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.filesystems.*;

/** Cleans up after an external compiler.
 *
 * @author rees0234
 */
public class XMLCCleanCompiler extends Compiler {

    private FileObject fo;

    public XMLCCleanCompiler(FileObject fo)
{
        this.fo = fo;
    }

    // This is important to write correctly!
    public boolean equals (Object o) {
        if (! (o instanceof XMLCCleanCompiler)) return false;
        XMLCCleanCompiler other = (XMLCCleanCompiler) o;
        // Add any other equality comparisons you may need,
        // according to the instance variables:
	try {
	    return fo.getFileSystem ().getSystemName ().equals (other.fo.getFileSystem ().getSystemName ()) &&
		fo.getPackageNameExt ('/', '.').equals (other.fo.getPackageNameExt ('/', '.'));
	} catch (FileStateInvalidException fsie) {
	    if (Boolean.getBoolean ("netbeans.debug.exceptions"))
		fsie.printStackTrace ();
	    return false;
	}
    }

    // Also, this must be the same for equal compilers!
    public int hashCode () {
        return 1234 ^ fo.getPackageNameExt ('/', '.').hashCode ();
    }

    /*
    // Only needed if you wish to automatically break apart
    // these compilers into different groups:
    public Object compilerGroupKey () {
	List l = new ArrayList (2);
	l.add (super.compilerGroupKey ());
	l.add (someThingUniqueToMe);
	return l;
    }
    */

    public Class compilerGroupClass () {
        return XMLCCleanCompilerGroup.class;
    }

    public boolean isUpToDate () {
        // Just check if there is anything to clean.
        return FileUtil.findBrother (fo, "compiledExt") == null;
    }

    // For use by the compiler group:
    public FileObject getFileObject () {
        return fo;
    }

}
