/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.services;

import java.util.*;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.execution.NbProcessDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

public class XMLCCompiler extends ExternalCompiler {

    private boolean building;
    private String myOpt;  //options to pass???

        /** Value of my compiler type */
    private XMLCCompilerType myType;

    /** Type of compilation */
//    protected Object type;

    /** Target filesystem, initially null.
     */
 //   private FileSystem target;

    /** @return XMLCCompilerType
    */
    XMLCCompilerType getMyType() {
        return myType;
    }

    /** Create an external compiler.
    * @param fo a file to compile
    * @param type the type of compilation ({@link #COMPILE}, {@link #BUILD}, or {@link #CLEAN})
    * @param nbDescriptor a description of an external compiler executable
    * @param err a regular expression to scan for compiler errors
    * @exception IllegalArgumentException if the file object is invalid
    */
/*    public XMLCCompiler(FileObject fo, Object type,
        NbProcessDescriptor nbDescriptor, ErrorExpression err,
        XMLCCompilerType ctype, FileSystem targetFS) {
        super (fo, type, nbDescriptor, err);
        this.myType = ctype;
        this.type = type;
        this.target = targetFS;
    }*/

    public XMLCCompiler(boolean building, FileObject primFile, NbProcessDescriptor compiler, ExternalCompiler.ErrorExpression errExpr, String myOpt) {
        super (primFile, building ? BUILD : COMPILE, compiler, errExpr);
        this.building = building;
        this.myOpt = myOpt;  //???
    }

    /** @return <tt>true</tt> if the compilers are equal */
/*    public boolean equals(Object o) {
        if (o instanceof XMLCCompiler) {
            JExternalCompiler him = (JExternalCompiler) o;
            return (super.equals(o) &&
                    (him.getFileObject() == getFileObject()) &&
                    (him.myType == myType));
        } else {
            return false;
        }
    }    */

    // This is important to write correctly!
    public boolean equals (Object o) {
        if (! super.equals (o)) return false;
        if (! (o instanceof XMLCCompiler)) return false;
        XMLCCompiler other = (XMLCCompiler) o;
        // Add any other equality comparisons you may need,
        // according to the instance variables:
        return myOpt.equals (other.myOpt) &&
               building == other.building;
    }

    // Also, this must be the same for equal compilers!
    public int hashCode () {
        return 9876 ^ getFileObject ().getPackageNameExt ('/', '.').hashCode ();
    }

    // Store this information for use by the compiler group.
    public String getMyOpt () {
        return myOpt;
    }

    public boolean isUpToDate () {
        // Always recompile when building.
        if (building) return false;
        // Otherwise, check neighboring files, e.g., for an extension
        // produced only by the compiler.
        FileObject fo = getFileObject ();
        FileObject compiled = FileUtil.findBrother (fo, "compiledExt");
        // If no compiled file, then of course it is not up-to-date.
        if (compiled == null) return false;
        // Else, check timestamps.
        return compiled.lastModified ().after (fo.lastModified ());
    }

    // Just specify the kind of compiler group to use. Its default constructor will be called.
    public Class compilerGroupClass () {
        return XMLCCompilerGroup.class;
    }

    // Make sure these compilers are split into separate groups if they do not
    // agree on the value of myOpt.
    public Object compilerGroupKey () {
        List l = new ArrayList (2);
        l.add (super.compilerGroupKey ());
        l.add (getMyOpt ());
        return l;
    }

}
