/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */
package org.enhydra.kelp.forte.services;
//
import org.openide.TopManager;
import org.openide.compiler.CompilerType;
import org.openide.cookies.CompilerCookie;
import org.openide.loaders.CompilerSupport;
import org.openide.loaders.DataObject;
import org.openide.loaders.MultiDataObject.Entry;

/**
 *
 * @author  rees0234
 * @version
 */
public class XMLCCompilerSupport extends CompilerSupport {
    private Entry sourceEntry;

    /**
     * Creates new XMLCCompilerSupport
     */
    protected XMLCCompilerSupport(Entry entry, Class cookie) {
        super(entry, cookie);
        sourceEntry = entry;
    }

    protected final Entry getSourceEntry() {
        return sourceEntry;
    }

    /**
     * @return default CompilerType for XMLCDOs
     */
    protected CompilerType defaultCompilerType() {

        // return getOpts().getCompiler();
        return (CompilerType) TopManager.getDefault().getServices().find(XMLCCompilerType.class);
    }

    /**
     * getter for JavaSettings
     */

    /*
     * private static JavaSettings getOpts() {
     * if (opts == null) {
     * opts = (JavaSettings) JavaSettings.findObject(JavaSettings.class, true);
     * }
     * return opts;
     * }
     */

    /**
     * Determines whether the source was successfully compiled. It uses
     * the configured CompilerType for determining what files should it look
     * after.
     */

    public boolean isUpToDate() {

        // quick: modified dataobjects cannot be up to date.
        DataObject d = getSourceEntry().getDataObject();

        if (d.isModified()) {
            return false;
        }
        CompilerCookie ccookie =
            (CompilerCookie) d.getCookie(CompilerCookie.Compile.class);

        if (ccookie == null) {
            return false;
        }
        org.openide.compiler.CompilerJob j =
            new org.openide.compiler.CompilerJob(org.openide.compiler.Compiler.DEPTH_ONE);

        return j.isUpToDate();
    }

    /**
     * Compile cookie support.
     * Note that as a special case, when {@link Compiler#DEPTH_ONE} is requested,
     * a {@link CompilerCookie.Build} will actually be sent to the compiler manager,
     * rather than a {@link CompilerCookie.Compile}, on the assumption that the user
     * wished to force (re-)compilation of the single data object.
     */

    public static class Compile extends XMLCCompilerSupport
        implements CompilerCookie.Compile {

        /**
         * New support for given entry. The file is taken from the
         * entry and is updated if the entry moves or renames itself.
         * @param entry entry to create instance from
         */
        public Compile(Entry entry) {
            super(entry, CompilerCookie.Compile.class);
        }
    }

    /**
     * Build cookie support.
     */
    public static class Build extends XMLCCompilerSupport
        implements CompilerCookie.Build {

        /**
         * New support for given entry. The file is taken from the
         * entry and is updated if the entry moves or renames itself.
         * @param entry entry to create instance from
         */
        public Build(Entry entry) {
            super(entry, CompilerCookie.Build.class);
        }

    }

    /**
     * Clean cookie support.
     */
    public static class Clean extends XMLCCompilerSupport
        implements CompilerCookie.Clean {

        /**
         * New support for given entry. The file is taken from the
         * entry and is updated if the entry moves or renames itself.
         * @param entry entry to create instance from
         */
        public Clean(Entry entry) {
            super(entry, CompilerCookie.Clean.class);
        }

    }
}
