/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

import org.openide.actions.*;
import org.openide.cookies.*;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileStateInvalidException;

import org.enhydra.kelp.forte.DeploySettings;

import org.enhydra.kelp.common.PathUtil;

import java.io.File;

/** Represents a Deploy object in the Repository.
 *
 * @author  rees0234
 */
public class DeployDataObject extends MultiDataObject {

    static final long serialVersionUID = -1461825550268566994L;

    protected DeploySettings settings = null;
    
    public DeployDataObject (FileObject pf, DeployDataLoader loader) throws DataObjectExistsException
    {
        super (pf, loader);
        init ();
    }

    private void init () {
        settings = DeploySettings.getDefault();
        CookieSet cookies = getCookieSet ();
        // Add whatever capabilities you need, e.g.:
        /*
        cookies.add (new ExecSupport (getPrimaryEntry ()));
        cookies.add (new EditorSupport (getPrimaryEntry ()));
        cookies.add (new CompilerSupport.Compile (getPrimaryEntry ()));
        cookies.add (new CompilerSupport.Build (getPrimaryEntry ()));
        cookies.add (new CompilerSupport.Clean (getPrimaryEntry ()));
        cookies.add (new OpenCookie () {
		public void open () {
		    // do something...but usually you want to use OpenSupport instead
		}
	    });
        */
        cookies.add(new XMLCEditorSupport(this));
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you add context help, change to:
        // return new HelpCtx (DeployDataObject.class);
    }

    protected Node createNodeDelegate () {
        return new DeployDataNode (this);
    }

    public String getPath()
    {
        FileObject file = getPrimaryFile();
        StringBuffer res = new StringBuffer();
        FileSystem fs = null;
        try
        {
           fs = file.getFileSystem();
        } catch (FileStateInvalidException e)
        {
           System.err.println(e);
           return "invalid";
        }

        res.append(fs.getSystemName());
        String pack = file.getPackageNameExt(File.separatorChar, '.');
        res.append(File.separatorChar);
        res.append(pack);

        return res.toString();
    }

    public String getProperty(String key)
    {
       String path = PathUtil.compressPathRelativeToProject(new ForteProject(), getPath());
       return (String)settings.getNodeProperty(path + key);    
    }

    public void setProperty(String key, String value)
    {
        String path = PathUtil.compressPathRelativeToProject(new ForteProject(), getPath());
        settings.setNodeProperty(path + key, value);
    }    
}
