/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;

// Forte imports
//import com.borland.jbuilder.node.JBProject;  ?????
import org.openide.loaders.DataNode;
import org.openide.nodes.Node;
import org.openide.nodes.FilterNode;
import org.openide.cookies.ProjectCookie;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.loaders.DataShadow;
import org.openide.loaders.DataFolder;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.filesystems.LocalFileSystem;
//Dusan
import org.openide.NotifyDescriptor;
import org.openide.TopManager;


// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterProject;

import org.enhydra.kelp.forte.XMLCSettings;
import org.enhydra.kelp.forte.DeploySettings;

// Standard imports
import java.io.File;
import javax.swing.SwingUtilities;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.net.URL;
import java.net.MalformedURLException;
/**
 *
 * @author  rees0234
 * @version
 */
public class ForteNode implements OtterFileNode
{
    protected DataObject nativeNode = null;
    private Throwable exception = null;

    /**
     * Constructor declaration
     *
     * @param n
     */
    public ForteNode(DataObject dob)
    {
        nativeNode = dob;
    }

    /**
     * Constructor declaration
     *
     * @param otterNode
     * @param path
     */
    public ForteNode(OtterNode parent, String path) //this is default for files
    {
        setNativeNode(FindData(path));
    }

    protected DataObject FindData(String path)
    {//not thrilled with this one....
        DataObject data = null;
        FileObject file = null;

        FileSystem fs = null;
        try {
            fs = ((ForteProject)getProject()).getRootFolder().getFileSystem();
        } catch (FileStateInvalidException e){
            System.err.println(e);
        }
        String adjPath = path;
        String sysVal = fs.getSystemName();
        if (path.startsWith(sysVal))
            adjPath = path.substring(sysVal.length()+ 1);
        file = fs.findResource(adjPath);
        if (file != null)
            try
            {
                data = DataObject.find(file);
            }catch (DataObjectNotFoundException e)
            {
                System.err.println(e);
            }
            return data;
    }


    public OtterProject getProject()
    {
        return new ForteProject();
    }

    /**
     * Method declaration
     *
     * @return
     */
    public String getFilePath()
    {
        if ((nativeNode != null))
        {
            if (nativeNode instanceof DataShadow)
                nativeNode = ((DataShadow)nativeNode).getOriginal();
            if (nativeNode != null)
            {
                FileSystem fs = null;
                StringBuffer res = new StringBuffer();
                FileObject file = nativeNode.getPrimaryFile();
                try
                {
                    fs = file.getFileSystem();
                } catch (FileStateInvalidException e)
                {
                    System.err.println(e);
                    return "invalid";
                }

                res.append(fs.getSystemName());

                String pack = new String();

                if (nativeNode instanceof DataFolder)
                {
                    pack = file.getPackageName(File.separatorChar);

                }
                else
                {
                    pack = file.getPackageNameExt(File.separatorChar, '.');
                }
                if (pack.length() > 0)
                {
                    res.append(File.separatorChar);
                    res.append(pack);
                }

                return res.toString();
            }

        }

        return null;  //should only return null on Project node
    }

    public String getXMLCOptionFilePath()
    {
        //this does not make sense for base node...only for XMLC-oriented nodes
        return null;
    }

    public void setXMLCOptionFilePath(String n)
    {
        //this does not make sense for base node...only for XMLC-oriented nodes
    }

    public String getXMLCParameters()
    {
        //this does not make sense for base node...only for XMLC-oriented nodes
        return null;
    }

    public void setXMLCParameters(String in)
    {

        //this does not make sense for base node...only for XMLC-oriented nodes
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     */
    public String getProperty(String property)
    {
        String path = PathUtil.compressPathRelativeToProject(this, getFilePath());

        if (property.startsWith("enhydra.xmlc"))
            return XMLCSettings.getDefault().getNodeProperty(path + property);
        else
            return DeploySettings.getDefault().getNodeProperty(path + property);
    }

    /**
     * Method declaration
     *
     * @param property
     * @param i
     */
    public void setProperty(String property, String value)
    {

        String path = PathUtil.compressPathRelativeToProject(this, getFilePath());

        if (property.startsWith("enhydra.xmlc"))
            XMLCSettings.getDefault().setNodeProperty(path + property, value);
        else
            DeploySettings.getDefault().setNodeProperty(path + property, value);
    }
    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     */

    public void setProperty(String property, int i)
    {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeNode()
    {
        return nativeNode;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setNativeNode(Object n)
    {
        nativeNode = (DataObject) n;
    }

    public OtterNode getParent()
    {
        DataFolder nativeParent = null;

        nativeParent = nativeNode.getFolder();
        if (nativeParent == null) //it its null, it is root of filesystem, which means parent must be project
        {
            return new ForteProject();
        }
        else
        {
            return new ForteFolderNode(nativeParent);
        }
    }

    public boolean isSelected()
    {
        return false;
    }

    public void setSelected(boolean b)
    {
        System.out.println("wrong setSelected");
    }

    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Throwable e)
    {
        exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Throwable getException()
    {
        return exception;
    }

    public void save()
    {
        //this should be handled by loader ?
    }


}
