/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ToolException;

// Forte imports
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.nodes.Children;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.loaders.DataShadow;
import org.openide.filesystems.FileObject;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.ExecSupport;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.TopManager;
import org.openide.execution.NbClassPath;

import org.netbeans.modules.text.TXTDataObject;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.deployer.RunParameters;

import org.enhydra.kelp.forte.XMLCSettings;
import org.enhydra.kelp.forte.DeploySettings;
//import org.enhydra.kelp.forte.DodsSettings;
import org.enhydra.kelp.forte.services.KelpExecutor;

import org.enhydra.tool.common.ToolException;

// Standard imports
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

//dr NetBeans imports
import org.openide.NotifyDescriptor;

/**
 * Class declaration
 *
 *
 * @author
 */
public class ForteProject extends OtterProject {

//dr
//    private Node nativeProject = null;
//    private ForteNodeFactory factory = null;
    public Node nativeProject = null;
    public ForteNodeFactory factory = null;

    private static final String SOURCE_PATH_ARRAY = "project.sourcepath.array";

    /**
     * Constructor declaration
     *
     *
     * @param project
     */
    public ForteProject () {
        nativeProject = TopManager.getDefault().getPlaces().nodes().projectDesktop();
        factory = new ForteNodeFactory();
    }

    public ForteProject(Node target)
    {
        nativeProject = target;
        factory = new ForteNodeFactory();
    }

    public OtterNodeFactory getNodeFactory() {
        return factory;
    }

    public String getGenerateToDirectory() {
	return getClassOutputPath();
    }

    public String getRootPath()
    {
        return XMLCSettings.getDefault().getRoot();
    }

    public String getDefaultRoot()
    {
        String root = null;
        Children kids = nativeProject.getChildren();
        if (kids == null)
            return null;
        Enumeration nodes = kids.nodes();

        while (nodes.hasMoreElements())
        {
            Node nextNode = (Node)nodes.nextElement();
            DataObject dob = (DataObject)nextNode.getCookie(DataObject.class);
            if (dob instanceof DataShadow)
               dob = ((DataShadow)dob).getOriginal();
            if (dob instanceof DataFolder)
            {
               FileObject file = dob.getPrimaryFile();
               if (file != null)
               {
                    String fileName = file.getPackageName(File.separatorChar);
                    FileSystem fs = null;
                    StringBuffer buf = null;
                    try {
                        fs = file.getFileSystem();
                    } catch (FileStateInvalidException e) {
                        System.err.println(e);
                    }
                    if (fs != null)
                    {
                        buf = new StringBuffer(fs.getSystemName());
                        if ((fileName != null) && (fileName.length() > 0))
                        {
                             buf.append(File.separatorChar).append(fileName);
                        }
                        root = buf.toString();
                        break;
                    }
                 }
             }

        }
        if ((root != null) && (root.length() > 0))
           return root;
       else
           return null;
    }

    public FileObject getRootFolder()
    {
        FileSystem[] fs = getFileSystems();
        FileObject root = null;
        String path = getRootPath();
        for (int i = 0; i < fs.length; i++)
        {
            FileObject sysRoot = fs[i].getRoot();
            String name = fs[i].getSystemName();
            if (name.equals(path))
                root = sysRoot;
            else if (path.startsWith(name))
                root = sysRoot.getFileObject(path.substring(name.length() + 1));
            if (root != null)
                break;
        }

        return root;
    }

    public boolean inProject(String path)
    {
        FileSystem[] fs = getFileSystems();
        FileObject target = null;
        for (int i = 0; i < fs.length; i++)
        {
            FileObject sysRoot = fs[i].getRoot();
            String name = fs[i].getSystemName();
            if (name.equals(path))
                target = sysRoot;
            else if (path.startsWith(name))
                target = sysRoot.getFileObject(path.substring(name.length() + 1));
            if (target != null)
                return true;
        }
        return false;
    }

    public Object getNativeNode()
    {
        return nativeProject;
    }

    public void setNativeNode(Object node)
    {
        nativeProject = (Node)node;
    }

    public OtterProject getProject()
    {
        return this;  //unless we support nested projects
    }

    public OtterNode getParent()
    {
        return null;  //unless we support nested projects
    }

    public FileSystem[] getFileSystems()
    {
        Vector systems = new Vector();

        Children kids = nativeProject.getChildren();
        Enumeration nodes = kids.nodes();
        while (nodes.hasMoreElements())
        {
            Node nextNode = (Node)nodes.nextElement();
            DataObject dob = (DataObject)nextNode.getCookie(DataObject.class);
            if (dob instanceof DataShadow)
                dob = ((DataShadow)dob).getOriginal();
            if (dob instanceof DataFolder)
            {
                FileObject file = dob.getPrimaryFile();
                try {
                    FileSystem fs = file.getFileSystem();
                    systems.add(fs);
                }catch (FileStateInvalidException e) {
                    System.err.println(e);
                }
            }
        }

        return (FileSystem[])systems.toArray(new FileSystem[0]);

    }


    public void save()
    {
    }

    public String getCodeGenDefaultDestination()
    {
        //for now - get the parent of first directory in the project
        String dest = new String();
        File   file = null;

        file = new File(getRootPath());
        dest = file.getParent();

        return dest;
    }

    /**
     * Find and return first .policy node found in project.
     */
    public OtterTextFileNode getFirstPolicy()
    {
        PolicyNodeReader reader = new PolicyNodeReader();
        return reader.getFirstNode(nativeProject);
    }

    /**
     * Get working directory for project run.
     */
    public String getWorkingPath()
    {
        String path = new String();

        path = getProperty(PropertyKeys.NAME_WORKING_DIR);
        if (path == null || path.length() == 0)
            path = getRootPath();
        return path;
    }

    /**
     * Set working directory for project run.
     */
    public void setWorkingPath(String path)
    {
       setProperty(PropertyKeys.NAME_WORKING_DIR, path);
    }

    public void configureRunClass()
    {
        String easRoot = ToolBoxInfo.getEnhydraRoot();
        String srcDir = XMLCSettings.getDefault().getSource();
        char sep = File.separatorChar;
        char psep = File.pathSeparatorChar;
        RunParameters rp = new RunParameters(this);

        FileObject root = getRootFolder();
        FileObject src = root.getFileObject(srcDir);
        if (src == null)
        {
            System.out.println("can't get source dir");
            return;
        }
        src.refresh();
        FileObject file = src.getFileObject("StartServer", "java");
        if (file == null)
        {
            return;
        }
        MultiDataObject dob = null;
        try {
                dob = (MultiDataObject)DataObject.find(file);
        } catch (DataObjectNotFoundException e) {
            System.err.println(e);
            return;
        }

        StringBuffer buf = new StringBuffer();
        KelpExecutor runExec = new KelpExecutor();
		//DACHA add 17.05.2003
		buf.append(' ');
		buf.append(rp.getVMParameters());
		buf.append(' ');
		//END DACHA
        buf.append("-cp \"");
//        buf.append(getSourcePath()); // vl
//        buf.append(psep); // vl
        buf.append(getSourcePath()); // vl
        buf.append(File.pathSeparatorChar); // vl
        buf.append(getJarPath()); // vl
        buf.append(File.pathSeparatorChar); // vl
//        buf.append(psep); // vl
        buf.append(easRoot);
        buf.append(sep);
        buf.append("lib");
        buf.append(sep);
        buf.append("build");
        buf.append(sep);
        buf.append("toolbox.jar");
        buf.append('\"');
        buf.append(' ');
        buf.append(rp.getTCcpArg());
        buf.append(' ');
        buf.append(rp.getPolicyArg());
        buf.append(' ');
        buf.append(rp.getBootArg());
        buf.append(" StartServer ");
        buf.append(rp.getAppParameters());

        runExec.setRunOption(buf.toString());
        try {
            ExecSupport.setExecutor(dob.getPrimaryEntry(), runExec);
        } catch (IOException e) {
            System.err.println(e);
        }

    }

    /**
     * Return all web.xml nodes.
     */
    public OtterTextFileNode[] getAllDeploymentDescs()
    {
        DeploymentNodeReader reader = new DeploymentNodeReader();
        return reader.getNodes(nativeProject);
    }

    /**
     * Return all nodes that ends with an extension in
     * the provided extension list.
     *
     * @param extensions
     */
    public OtterFileNode[] findFileNodesByType(String[] extensions)
    {
        NodeReader reader = new NodeReader(extensions);
        return reader.getNodes(nativeProject);
    }
    /**
     * Get all templates contained in the input directory.
     */
    public OtterFileNode[] getAllInput()
    {
        String[]    ext = new String[0];
        OtterFileNode[] input = new OtterFileNode[0];
        OtterFileNode[] nodes = new OtterFileNode[0];
        Vector      nodeVector = new Vector();
        PathHandle  inputPath = null;
        PathHandle  cursorPath = null;

        inputPath = PathHandle.createPathHandle(getDeployInputPath());
        if (isDeployInputFilter()) {
            ext = new String[1];
            ext[0] = Constants.TYPE_IN;
        }
        nodes = findFileNodesByType(ext);
        for (int i = 0; i < nodes.length; i++) {
            cursorPath = PathHandle.createPathHandle(nodes[i].getFilePath());
            if (inputPath.parentOf(cursorPath)) {
                nodeVector.addElement(nodes[i]);
            }
        }
        input = new OtterFileNode[nodeVector.size()];
        input = (OtterFileNode[]) nodeVector.toArray(input);
        nodeVector.clear();
        return input;
    }

    /**
     * Get all document nodes in the project.
     */
    public OtterDocumentNode[] getAllDocuments()
    {
        DocumentNodeReader reader = new DocumentNodeReader();
        return reader.getNodes(this);

    }

    /**
     * Return the IDE implementation of the project node.
     */
    public Object getNativeProject()
    {
        return nativeProject;
    }

    /**
     * Does IDE allow for integrated build.
     */
    public boolean isOpenBuild()
    {
        return false; //for now
    }

    /**
     * Method declaration
     *
     * @return
     */
    public String getClassOutputPath()
    {
        return getSourcePath();
    }

    public void setClassOutputPath(String p)
    {

    }

    //
    public boolean isDefaultProject()
    {
        return false;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public String getSourcePath()
    {
        StringBuffer buf = new StringBuffer();
        String src = XMLCSettings.getDefault().getSource();
        buf.append(getRootPath()).append(File.separatorChar).append(src);
/*
        buf.append(getRootPath()).append(File.separatorChar).append(src).append(File.pathSeparatorChar);
*/
        return buf.toString() ;
    }

    public String getJarPath()
    {
        StringBuffer buf = new StringBuffer();
        String src = XMLCSettings.getDefault().getSource();
        String jarName = getRootPath().substring(getRootPath().lastIndexOf('/')); // vl
        buf.append(getRootPath()).append(File.separatorChar).append("output").
        append(File.separatorChar).append("lib").append(jarName+".jar"); // vl
        return buf.toString() ;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public String[] getSourcePathArray()
    {
        String in = (String)XMLCSettings.getDefault().propertyValue(SOURCE_PATH_ARRAY);
        if (in != null && in.length() > 0)
            return PropUtil.listToArray(in, new String[0]);
        else
            return new String[] {getRootPath()};

/*        Children kids = nativeProject.getChildren();
        Node nodes[] = kids.getNodes();
        String paths[] = new String[nodes.length];
        for (int i = 0; i < nodes.length; i++)
        {
            DataObject dob = (DataObject)nodes[i].getCookie(DataObject.class);
            if (dob instanceof DataShadow)
                dob = ((DataShadow)dob).getOriginal();
            if (dob instanceof DataFolder)
            {
                FileObject file = dob.getPrimaryFile();
                try {
                    FileSystem fs = file.getFileSystem();
                    paths[i] = fs.getDisplayName();
                } catch (FileStateInvalidException e) {
                    System.err.println(e);
                }
            }

        }
        return paths;*/
    }

    /**
     * Method declaration
     *
     * @param pathAsArray
     */
    public void setSourcePathArray(String[] pathAsArray)
    {
        String out = PropUtil.arrayToList(pathAsArray);
        XMLCSettings.getDefault().setProperty(SOURCE_PATH_ARRAY, out);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getClassPath()
    {
        StringBuffer buf = new StringBuffer();

        //I think there's a better way to do this, but....
        //here are the paths added by user via the filesystem:
        Enumeration fss = TopManager.getDefault().getRepository().getFileSystems();
        while (fss.hasMoreElements ())
        {
            FileSystem fs = (FileSystem) fss.nextElement ();
            buf = buf.append(fs.getSystemName());
            buf = buf.append(File.pathSeparatorChar);
        }
        //these are on the ide classpath and they are available to the project - which seems wierd
        buf.append(NbClassPath.createClassPath().getClassPath());
        return buf.toString();
    }

    public boolean cpContains(String target)
    {
        StringTokenizer tokenizer= new StringTokenizer(getClassPath(), File.pathSeparator);
        String cur = null;
        while (tokenizer.hasMoreTokens())
        {
            try{
                cur = tokenizer.nextToken();
            } catch (NoSuchElementException e) {
                break;
            }

            if (cur.equals(target))
                return true;
        }
        return false;
    }
    /**
     * Method declaration
     *
     * @param property
     *
     * @return
     */
    public String getProperty(String property)
    {
        if (property.startsWith("enhydra.xmlc"))
            return XMLCSettings.getDefault().propertyValue(property);
        else
            return DeploySettings.getDefault().propertyValue(property);
    }

    /**
     * Method declaration
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value)
    {
        if (property.startsWith("enhydra.xmlc"))
            XMLCSettings.getDefault().setProperty(property, value);
        else
            DeploySettings.getDefault().setProperty(property, value);
    }

    /**
     * Method declaration
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, int value)
    {
        setProperty(property, Integer.toString(value));
    }

    // override OtterProject
    public boolean isDeployStartupJavaReadOnly() {
       return true;
    }



   class DeploymentNodeReader {
        private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     */
        protected DeploymentNodeReader() {}

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
        */
       protected OtterTemplateNode[] getNodes(Node nativeProject) {
            OtterTemplateNode[] nodes = new OtterTemplateNode[0];

            Node project = nativeProject;
            nodeVector.clear();

            FileObject root = getRootFolder();
            if (root != null)
                readFiles(root);
            nodes = new OtterTemplateNode[nodeVector.size()];
            nodes = (OtterTemplateNode[])nodeVector.toArray(nodes);
            nodeVector.clear();
            return nodes;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void readFiles(FileObject container) {

            FileObject[] children = container.getChildren();
            for (int i = 0; i < children.length; i++)
            {
                if (children[i].isData())
                {
                    String ext = children[i].getExt();
                    if (((ext.equals(Constants.FILE_WEB_XML))
                           || (ext.equals(Constants.FILE_EJB_JAR_XML))))
                    {
                        ForteTemplateNode node = null;
                        try{
                            DataObject dob = DataObject.find(children[i]);
                            if (dob instanceof DeployDataObject)  //just to be sure
                                try {
                                    node = new ForteTemplateNode((DeployDataObject)dob);
                                } catch (ToolException e) {
                                   System.err.println(e);
                                }
                                if (node != null)
                                  nodeVector.add(node);
                            }
                        catch(DataObjectNotFoundException e) {
                                System.err.println(e);
                            }

                    }

                }
                else if (children[i].isFolder())
                    readFiles(children[i]);

            }
        }

    }

    class DocumentNodeReader {
        private Vector       nodeVector = new Vector();
        private Node project = null;
        private String[]     docTypes = new String[0];

        /**
         * Constructor declaration
         *
         */
        protected DocumentNodeReader() {}

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
         */
        protected OtterDocumentNode[] getNodes(OtterProject p) {
            OtterDocumentNode[] nodes = new OtterDocumentNode[0];
            project = (Node)p.getNativeNode();
            docTypes = p.getDocTypes();
            nodeVector.clear();
            FileObject root = getRootFolder();
            if (root != null)
                readFiles(root);
            nodes = new OtterDocumentNode[nodeVector.size()];
            nodes = (OtterDocumentNode[]) nodeVector.toArray(nodes);
            nodeVector.clear();
            return nodes;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void readFiles(FileObject container) {

            FileObject[] children = container.getChildren();
            for (int i = 0; i < children.length; i++)
            {
                if (children[i].isData())
                {
                    if (isTarget(children[i].getExt()))
                    {
                        try
                        {
                            DataObject dob = DataObject.find(children[i]);
                            if (dob instanceof XMLCDataObject)  //just to be sure
                                nodeVector.add(new ForteDocumentNode((XMLCDataObject)dob));
                        }
                        catch(DataObjectNotFoundException e) {
                            System.err.println(e);
                        }
                    }

                }
                else if (children[i].isFolder())
                    readFiles(children[i]);

            }
        }

        protected boolean isTarget(String ext)
        {
             for (int j = 0; j < docTypes.length; j++)
             {
                 if (ext.equals(docTypes[j]))
                    return true;
             }
             return false;
        }

    }

    class NodeReader {
        private Vector   nodeVector = new Vector();
        private String[] extensions = new String[0];

        /**
         * Constructor declaration
         *
         *
         * @param exts
         */
        protected NodeReader(String[] exts) {
           extensions = exts;
        }

        /**
        * Method declaration
        *
        *
        * @param nativeProject
         *
         * @return
         */
        protected OtterFileNode[] getNodes(Node nativeProject) {
            OtterFileNode[] nodes = new OtterFileNode[0];
            Node project = nativeProject;
            nodeVector.clear();
            FileObject root = getRootFolder();
            if (root != null)
                readFiles(root);
            nodes = new OtterFileNode[nodeVector.size()];
            nodes = (OtterFileNode[]) nodeVector.toArray(nodes);
            nodeVector.clear();
            return nodes;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void readFiles(FileObject container) {
            FileObject[] children = container.getChildren();
            for (int i = 0; i < children.length; i++)
            {
                if (children[i].isData())
                {
                    if (isTarget(children[i].getExt()) || (extensions.length == 0))
                    {
                        DataObject dob = null;
                        try
                        {
                            dob = DataObject.find(children[i]);
                        }
                        catch(DataObjectNotFoundException e) {
                            System.err.println(e);
                        }

                        if (dob instanceof XMLCDataObject)
                            nodeVector.add(new ForteDocumentNode((XMLCDataObject)dob));
                        else if (dob instanceof DeployDataObject)
                        {
                            try{
                                nodeVector.add(new ForteTemplateNode((DeployDataObject)dob));
                            } catch (ToolException e){
                                System.err.println(e);
                            }
                        }

                        else if (dob instanceof TXTDataObject)
                            nodeVector.add(new ForteTextFileNode((TXTDataObject)dob));
                        else
                            nodeVector.add(new ForteNode(dob));

                    }

                }
                else if (children[i].isFolder())
                    readFiles(children[i]);

            }
        }

        protected boolean isTarget(String ext)
        {
             for (int j = 0; j < extensions.length; j++)
             {
                 if (ext.equals(extensions[j]))
                    return true;
             }
             return false;
        }
    }

   class PolicyNodeReader {
        private OtterTextFileNode node = null;

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
         */
        protected OtterTextFileNode getFirstNode(Node nativeProject) {
            TXTDataObject target = null;

            FileObject root = getRootFolder();
            if (root != null)
                target = recursiveRead(root);
            if (target != null)
                return new ForteTextFileNode(target);
            else return null;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private TXTDataObject recursiveRead(FileObject container) {

            FileObject[] children = container.getChildren();

            for (int i = 0; i < children.length; i++)
            {

                if (children[i].isData())
                {
                    if (children[i].getExt().equals(Constants.FILE_JAVA_POLICY))
                    {
                        try{
                            DataObject dob = DataObject.find(children[i]);
                            return (TXTDataObject)dob;
                        } catch(DataObjectNotFoundException e) {
                            System.err.println(e);
                        }
                    }
                }

               if (children[i].isFolder())
                   return recursiveRead(children[i]);


            }
            return null;
        }
    }

    /**
    * Method declaration
    *
    *
    * @param container
    */
    public void setEnhydraPath() {
//Dusan 20.11.02.
        String enhydraPath = getClassPath();
        enhydraPath = enhydraPath.substring(0, enhydraPath.indexOf("enhydra.jar")-5);
 	if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
	  enhydraPath = enhydraPath.substring(enhydraPath.lastIndexOf(";")+1,
            enhydraPath.length());
         }
         else {
      	  enhydraPath = enhydraPath.substring(enhydraPath.lastIndexOf(":")+1,
            enhydraPath.length());
      	}

        setProperty(DODS_ENHYDRA_PATH, enhydraPath);
    }

}
