/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.build;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.deployer.DeployBuilder;

// AddinJBuilder
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// JBuilder
import com.borland.primetime.build.BuildTask;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Project;
import com.borland.jbuilder.node.JBProject;

// JDK
import java.io.File;
import java.util.ArrayList;
import java.util.ResourceBundle;


/**
 * BuildTask implementation for calling XMLC.
 *
 */
public class DeployBuildTask implements BuildTask, WriteListener {


    //
    static ResourceBundle      res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");    // nores
    //
    public static final Object SHARED_KEY = new Object();
    //
    private boolean            nodeOnly = true;
    private BuildProcess       buildProcess = null;
    private ArrayList          nodeList = new ArrayList();

    public DeployBuildTask() {}

    protected void setNodeOnly(boolean b) {
        nodeOnly = b;
    }

    protected boolean isNodeOnly() {
        return nodeOnly;
    }

    /**
     * Copy conf.in to output directory and replace placeholders.
     */
    public boolean build(BuildProcess buildProcess, Project project,
                         boolean rebuild) {


        this.buildProcess = buildProcess;
        OtterProject      otterProject =
            new PrimeProject((JBProject) project);
        int               nodeCount = nodeList.size();
        int               compileCount = nodeCount;
        OtterTemplateNode current = null;
        DeployBuilder builder = null;

        // remove upToDate nodes from vector if not a rebuild.
        if (!rebuild) {
            for (int i = 0; i < nodeCount; i++) {
                current = (OtterTemplateNode) nodeList.get(i);
                if (upToDate(current)) {
                    nodeList.set(i, new Object());
                    compileCount--;
                }
            }
        }

        OtterTemplateNode[] nodes = new OtterTemplateNode[compileCount];

        for (int i = 0; i < nodeCount; i++) {
            if (nodeList.get(i) instanceof OtterTemplateNode) {
                nodes[i] = (OtterTemplateNode) nodeList.get(i);
            }
        }
        WriteListener[] w = new WriteListener[1];

        w[0] = this;
        if (otterProject.isDeployBuild()&&!(compileCount==0)) {
            buildProcess.fireBuildStatus(res.getString("Processing_templates_"),
                                         true);
            builder = new DeployBuilder(this, true);
            builder.setProject(otterProject);
            if (isNodeOnly()) {
              builder.setInputNodes(nodes);
            }
            builder.setEcho(otterProject.isDeployEcho());
            builder.buildInCurrentThread();
        }
        return true;
    }

    /**
     * Build messages.
     *
     */
    public void onWrite(WriteEvent e) {
        if (buildProcess != null) {
            buildProcess.fireBuildMessage(e.getString(), e.getString());
        }
    }

    /**
     * Collect list of nodes to build.
     *
     */
    public void addNode(OtterTemplateNode n) {
        nodeList.add(n);
    }

    /**
     * Check if node up to date.
     *
     */
    private boolean upToDate(OtterTemplateNode templateNode) {
        boolean  built = false;
        FileNode fileNode = (FileNode) templateNode.getNativeNode();
        File outFile = new File(PathUtil.getOutputPath(templateNode));

        if (fileNode.isModified()) {
            try {
                fileNode.save();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else if (!outFile.exists()) {}
        else if (outFile.lastModified()
                 > fileNode.getUrl().getFileObject().lastModified()) {
            built = true;
        }
        return built;
    }

}
