/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

// ToolBox imports
import org.enhydra.tool.common.ToolException;

// JBuilder imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;
import com.borland.primetime.build.Phase;


// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.jbuilder.node.PrimeTemplateNode;
import org.enhydra.kelp.jbuilder.node.PrimeTextFileNode;
import org.enhydra.kelp.jbuilder.properties.DeployProperties;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class DeployBuilder extends Builder {
    public DeployBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        if (KelpInfo.isClassPathComplete()) {
            BuilderManager.registerBuilder(new DeployBuilder());
        }
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
                                   Node nativeNode) {

        DeployBuildTask   task = null;
        TextFileNode      nativeTextNode = null;
        OtterTemplateNode otterTemplateNode = null;
        OtterTextFileNode otterTextNode = null;
        OtterProject otterProject = null;
        String            stringIn = null;
        boolean           selected = false;
        if (nativeNode instanceof Project) {
            task = findTask(buildProcess);
            task.setNodeOnly(false);

        } else if (nativeNode instanceof TextFileNode) {
            nativeTextNode = (TextFileNode) nativeNode;
            otterProject =
                new PrimeProject((JBProject) nativeTextNode.getProject());
            otterTextNode = new PrimeTextFileNode(nativeTextNode);
            if (PathUtil.isTemplate(otterTextNode)) {
               try {
                 otterTemplateNode = new PrimeTemplateNode(nativeTextNode);
              } catch (ToolException e) {
                e.printStackTrace();
              }
            }
        }
        if (otterTemplateNode != null && otterTemplateNode.isSelected()) {
            task = findTask(buildProcess);
            task.addNode(otterTemplateNode);
      	if(otterProject.isDeployBuild()) {
				buildProcess.addDependency(DeployBuildTask.SHARED_KEY, Phase.DEPLOY_PHASE);
          }
        }
    }

    private DeployBuildTask findTask(BuildProcess buildProcess) {
      DeployBuildTask   task = null;
      //task = (DeployBuildTask) buildProcess.getTask(DeployBuildTask.SHARED_KEY);
      //kipz
      task = (DeployBuildTask)buildProcess.getFirstBuildTask(DeployBuildTask.SHARED_KEY, DeployBuildTask.class);
      if (task == null) {
         task = new DeployBuildTask();
         // There is both a map and a list
          //buildProcess.putTask(DeployBuildTask.SHARED_KEY, task);
          //buildProcess.getBuildTaskList().add(task);
         buildProcess.createTask(DeployBuildTask.class,DeployBuildTask.SHARED_KEY);
         return (DeployBuildTask)buildProcess.getFirstBuildTask(DeployBuildTask.SHARED_KEY, DeployBuildTask.class);
      }
      return task;
    }
}
