/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.properties;

// JBuilder imports
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.properties.PropertyPageFactory;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.properties.PropertyManager;
import com.borland.primetime.properties.PropertyGroup;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeTextFileNode;

// Standard imports
import java.io.File;
import java.util.ResourceBundle;
public class DeployProperties implements PropertyGroup {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res"); // nores

    public DeployProperties() {}

    /**
     * Initialize this OpenTool
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        if (KelpInfo.isClassPathComplete()) {
            PropertyManager.registerPropertyGroup(new DeployProperties());
        }
    }

    /**
     * Returns a PropertyPageFactory for the project or for an
     * conf.in file node.  Null if topic is something else.
     */
    public PropertyPageFactory getPageFactory(final Object topic) {
        PropertyPageFactory factory = null;
        TextFileNode        nativeTextNode = null;
        OtterTextFileNode   otterNode = null;

        if (topic instanceof TextFileNode) {
            nativeTextNode = (TextFileNode) topic;
            otterNode = new PrimeTextFileNode(nativeTextNode);
            if (PathUtil.isTemplate(otterNode)) {
                factory =
                    (new PropertyPageFactory("Input Template") {
                    public PropertyPage createPropertyPage() {
                        return new TemplateNodePropertyPage(topic);
                    }

                });
            }
        } else if (topic instanceof Project) {

            // Project property page to set your XMLC properties
            factory =
                (new PropertyPageFactory("Kelp Deployer") {
                public PropertyPage createPropertyPage() {
                    return new DeployProjectPropertyPage(topic);
                }

            });
        }
        return factory;
    }

    public void initializeProperties() {}

}
