package org.enhydra.kelp.eclipse.enhydraWizard;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageDeclaration;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.ClasspathEntry;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.preferences.NewJavaProjectPreferencePage;
import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.BuildPathsBlock;
import org.eclipse.jdt.ui.wizards.JavaCapabilityConfigurationPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardPage;
import org.enhydra.tool.ToolBoxInfo;

/**
 * @author Tweety
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class NewEnhydraWizardSecondPage
	extends JavaCapabilityConfigurationPage {

	private NewEnhydraWizardPage fMainPage;
	private IPath fCurrProjectLocation;
	private boolean fProjectCreated;
	private boolean firstUpdate = true;
	
	/**
	 * Constructor for NewEnhydraWizardSecondPage.
	 */
	public NewEnhydraWizardSecondPage() {
		super();

		fCurrProjectLocation= fMainPage.getLocationPath();
		fProjectCreated= false;
	}
	
	/**
	 * Constructor for ProjectWizardPage.
	 */
	public NewEnhydraWizardSecondPage(NewEnhydraWizardPage mainPage) {
		super();
		
		fMainPage= mainPage;
		fCurrProjectLocation= fMainPage.getLocationPath();
		fProjectCreated= false;
	}
	
	private boolean canDetectExistingClassPath(IPath projLocation) {
		return projLocation.toFile().exists() && !Platform.getLocation().equals(projLocation);
	}
	
	private void update() {
		IPath projLocation = fMainPage.getLocationPath();
		if (!projLocation.equals(fCurrProjectLocation) && canDetectExistingClassPath(projLocation)) {
			String title = NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.EarlyCreationDialog.title"); //$NON-NLS-1$
			String description = NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.EarlyCreationDialog.description"); //$NON-NLS-1$
			if (MessageDialog.openQuestion(getShell(), title, description)) {
				createAndDetect();
			}
		}
					
		fCurrProjectLocation = projLocation;
				
		IJavaProject prevProject = getJavaProject();
		IProject currProject = fMainPage.getProjectHandle();
		if ((prevProject == null) || !currProject.equals(prevProject.getProject())) {
			init(JavaCore.create(currProject), null, null, false);
		}
/**/		
		if (firstUpdate) {
			IClasspathEntry[] sourceDirectories = createSourceDirectories();
			IClasspathEntry[] myClasspaths = createMyClasspaths();
			IClasspathEntry[] allEntries = joinEntries(sourceDirectories, myClasspaths);
			
			try {
				init(getJavaProject(), new Path(getOutputLocation() + "/classes"), allEntries, false);
			} catch (NullPointerException e) {
				NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.CreatingEnhydraClasspathOperation.error");
			}
				
			firstUpdate = false;
		}
/**/
	}

	private void createAndDetect() {
		IRunnableWithProgress op= new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				if (monitor == null)
					monitor= new NullProgressMonitor();		
		
				monitor.beginTask(NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.EarlyCreationOperation.desc"), 3);				 //$NON-NLS-1$
				try {
					createProject(new SubProgressMonitor(monitor, 1));
					initFromExistingStructures(new SubProgressMonitor(monitor, 2));						
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				}
			}
		};

		try {
			getContainer().run(false, true, op);
		} catch (InvocationTargetException e) {
			String title= NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.EarlyCreationOperation.error.title"); //$NON-NLS-1$
			String message= NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.EarlyCreationOperation.error.desc");			 //$NON-NLS-1$
			ExceptionHandler.handle(e, getShell(), title, message);
		} catch  (InterruptedException e) {
			// cancel pressed
		}
	}
		
	/* (non-Javadoc)
	 * @see IDialogPage#setVisible(boolean)
	 */
	public void setVisible(boolean visible) {
		if (visible) {
			update();
		}
		super.setVisible(visible);
	}
	
	/* (non-Javadoc)
	 * @see IWizardPage#getPreviousPage()
	 */
	public IWizardPage getPreviousPage() {
		if (fProjectCreated) {
			return null;
		}
		return super.getPreviousPage();
	}
	
	public IRunnableWithProgress getRunnable() {
		return new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				if (monitor == null)
					monitor= new NullProgressMonitor();
				
				monitor.beginTask(NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.NormalCreationOperation.desc"), 4); //$NON-NLS-1$
				try {
					createProject(new SubProgressMonitor(monitor, 1));
					if (getJavaProject() == null) {
						initFromExistingStructures(new SubProgressMonitor(monitor, 1));
					} else {
						monitor.worked(1);
					}
					configureJavaProject(new SubProgressMonitor(monitor, 2));
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
	}

	private void createProject(IProgressMonitor monitor) throws CoreException {
		IProject project= fMainPage.getProjectHandle();
		IPath projectLocation= fMainPage.getLocationPath();
		BuildPathsBlock.createProject(project, projectLocation, monitor);
		fProjectCreated= true;
	}
	
	private void initFromExistingStructures(IProgressMonitor monitor) throws CoreException {
		if (monitor == null)
			monitor= new NullProgressMonitor();		
		
		monitor.beginTask(NewEnhydraWizardMessages.getString("NewEnhydraWizardSecondPage.DetectingClasspathOperation.desc"), 2); //$NON-NLS-1$
		try {
			IProject project= fMainPage.getProjectHandle();
			
			if (project.getFile(".classpath").exists()) { //$NON-NLS-1$
				init(JavaCore.create(project), null, null, false);
				monitor.worked(2);
			} else{
				final HashSet sourceFolders= new HashSet();
				IResourceVisitor visitor= new IResourceVisitor() {
					public boolean visit(IResource resource) throws CoreException {
						return doVisit(resource, sourceFolders);
					}
				};
				project.accept(visitor);
				monitor.worked(1);
								
				IClasspathEntry[] entries= null;
				IPath outputLocation= null;
				
				if (!sourceFolders.isEmpty()) {
					int nSourceFolders= sourceFolders.size();
					IClasspathEntry[] jreEntries= NewJavaProjectPreferencePage.getDefaultJRELibrary();
					entries= new IClasspathEntry[nSourceFolders + jreEntries.length];
					Iterator iter = sourceFolders.iterator();
					for (int i = 0; i < nSourceFolders; i++) {
						entries[i]= JavaCore.newSourceEntry((IPath) iter.next());
					}
					System.arraycopy(jreEntries, 0, entries, nSourceFolders, jreEntries.length);
					
					IPath projPath= project.getFullPath();
					if (nSourceFolders == 1 && entries[0].getPath().equals(projPath)) {
						outputLocation= projPath;
					} else {
						outputLocation= projPath.append(NewJavaProjectPreferencePage.getOutputLocationName());
					} 				
					if (!JavaConventions.validateClasspath(JavaCore.create(project), entries, outputLocation).isOK()) {
						outputLocation= null;
						entries= null;
					}
				}
				init(JavaCore.create(project), outputLocation, entries, false);
				monitor.worked(1);
			}
		} finally {
			monitor.done();
		}
		
	}
	
	private boolean doVisit(IResource resource, HashSet sourceFolders) throws JavaModelException {
		if (!sourceFolders.isEmpty()) {
			IResource curr= resource;
			while (curr.getType() != IResource.ROOT) {
				if (sourceFolders.contains(curr.getFullPath())) {
					return false;
				}
				curr= curr.getParent();
			}
		}
		if (resource.getType() == IResource.FILE) {
			if ("java".equals(resource.getFileExtension())) { //$NON-NLS-1$
				ICompilationUnit cu= JavaCore.createCompilationUnitFrom((IFile) resource);
				if (cu != null) {
					IPath packPath= resource.getParent().getFullPath();
					IPackageDeclaration[] decls= cu.getPackageDeclarations();
					if (decls.length == 0) {
						sourceFolders.add(packPath);
					} else {
						IPath relpath= new Path(decls[0].getElementName().replace('.', '/'));
						int remainingSegments= packPath.segmentCount() - relpath.segmentCount();
						if (remainingSegments >= 0) {
							IPath prefix= packPath.uptoSegment(remainingSegments);
							IPath common= packPath.removeFirstSegments(remainingSegments);
							if (common.equals(relpath)) {
								sourceFolders.add(prefix);
							}
						}
					}
				}
			}
		}
		return true;
	}

	
	/**
	 * Called from the wizard on cancel.
	 */
	public void performCancel() {
		if (fProjectCreated) {
			try {
				fMainPage.getProjectHandle().delete(false, false, null);
			} catch (CoreException e) {
				JavaPlugin.log(e);
			}
		}
	}
	
	
	/**
	 * Initializes the page with the project and default classpaths.
	 * <p>
	 * The default classpath entries must correspond the the given project.
	 * </p>
	 * <p>
	 * The caller of this method is responsible for creating the underlying project. The page will create the output,
	 * source and library folders if required.
	 * </p>
	 * <p>
	 * The project does not have to exist at the time of initialization, but must exist when executing the runnable
	 * obtained by <code>getRunnable()</code>.
	 * </p>
	 * @param project The Java project.
	 * @param entries The default classpath entries or <code>null</code> to let the page choose the default
	 * @param path The folder to be taken as the default output path or <code>null</code> to let the page choose the default
	 * @return overrideExistingClasspath If set to <code>true</code>, an existing '.classpath' file is ignored. If set to <code>false</code>
	 * the given default classpath and output location is only used if no '.classpath' exists.
	 */
	public void init(IJavaProject jproject, IPath defaultOutputLocation, IClasspathEntry[] defaultEntries, boolean defaultsOverrideExistingClasspath) {
		super.init(jproject, defaultOutputLocation, defaultEntries, defaultsOverrideExistingClasspath);
	}
	
	public IClasspathEntry[] createMyClasspaths() {

    final int jarFileElements = 23;
		IClasspathEntry[] classpaths = getRawClassPath();
		// Removed entry[0] - default source directory (Project root) make problems due project rebuild
// vl
//		IClasspathEntry[] myClasspaths = new IClasspathEntry[classpaths.length + 21 - 1];
		IClasspathEntry[] myClasspaths = new IClasspathEntry[classpaths.length + jarFileElements - 1]; // vl
		
		for(int i = 1; i < classpaths.length; i++) {
			myClasspaths[i-1] = classpaths[i];
		}

		
		String enh_home = ToolBoxInfo.getEnhydraRoot();
		String java_home = System.getProperty("java.home");

		int i = classpaths.length - 1;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/log4j.jar"), null, null, false);
		
		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(java_home+"/../lib/tools.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/admin.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/core.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/util.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/dods/lib/dods-runtime.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/enhydra.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/gnu-regexp.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/jtidy.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/servlet.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/snoop.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/tomcat.jar"), null, null, false);
/* vl 19.07.03
		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/wireless.jar"), null, null, false);
*/
		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/dom/xmlc-chtml.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/dom/xmlc-voicexml.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/dom/xmlc-wml.jar"), null, null, false);
// vl 19.07.03

		i++;
//		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
// vl 20.07.03
//		ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/xerces.jar"), null, null, false);
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
		ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/xercesImpl.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
		ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/xml-apis.jar"), null, null, false);
// vl 20.07.03

/* vl 19.07.03
		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/xhtml.jar"), null, null, false);
*/
		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/dom/xhtml.jar"), null, null, false);
// vl 19.07.03

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/xmlc.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/xmlcSupport.jar"), null, null, false);

		//SV i++;
		//SV myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			//SVClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/tools/Xalan/xalan.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/lib/build/toolbox.jar"), null, null, false);

		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/dods/lib/dods.jar"), null, null, false);
			
		i++;
		myClasspaths[i] = new ClasspathEntry(IPackageFragmentRoot.K_SOURCE,
			ClasspathEntry.CPE_LIBRARY, new Path(enh_home + "/dods/lib/ejen.jar"), null, null, false);

 
 		return myClasspaths;
		
	}
	
	
	private IClasspathEntry[] createSourceDirectories() {
		final HashSet sourceFolders = new HashSet();
		IClasspathEntry[] entries = null;
		IPath outputLocation = null;
		IProject project = fMainPage.getProjectHandle();
		IPath projPath = project.getFullPath();
				
		if (!sourceFolders.isEmpty()) {
			int nSourceFolders = sourceFolders.size();
			IClasspathEntry[] jreEntries = NewJavaProjectPreferencePage.getDefaultJRELibrary();
			entries = new IClasspathEntry[nSourceFolders + jreEntries.length];
			Iterator iter = sourceFolders.iterator();
			for (int i = 0; i < nSourceFolders; i++) {
				entries[i] = JavaCore.newSourceEntry((IPath) iter.next());
			}
			System.arraycopy(jreEntries, 0, entries, nSourceFolders, jreEntries.length);
			
			
			if (nSourceFolders == 1 && entries[0].getPath().equals(projPath)) {
				outputLocation = projPath;
			} else {
				outputLocation = projPath.append(NewJavaProjectPreferencePage.getOutputLocationName());
			} 				
			if (!JavaConventions.validateClasspath(JavaCore.create(project), entries, outputLocation).isOK()) {
				outputLocation= null;
				entries= null;
			}
			
		} else {
			entries = new IClasspathEntry[0];
		}

		String projDirName = projPath.segment(projPath.segmentCount() - 1);
		IClasspathEntry[] myEntries = new IClasspathEntry[entries.length + 3];
		myEntries[entries.length] = JavaCore.newSourceEntry((IPath) new Path("/" + projDirName + "/src"));
		myEntries[entries.length + 1] = JavaCore.newSourceEntry((IPath) new Path("/" + projDirName + "/xmlc_generated_src"));
		myEntries[entries.length + 2] = JavaCore.newSourceEntry((IPath) new Path("/" + projDirName + "/dods_generated_src"));

		return myEntries;
	}
	
	private IClasspathEntry[] joinEntries(IClasspathEntry[] a, IClasspathEntry[] b) {
		
		IClasspathEntry[] result;
		try {
			result = new IClasspathEntry[a.length + b.length];
			for(int i = 0; i < a.length; i++) {
				result[i] = a[i];
			}
			for(int i = a.length; i < a.length + b.length; i++) {
				result[i] = b[i - a.length];
			}
		} catch (NullPointerException e) {
			if(a == null) {
				if(b == null) return null;
				else return b;
			} else {
				if(b == null) return null;
				else return a;
			}
		}

		return result;
	}
	

}
