/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

import org.enhydra.kelp.ant.node.AntProject;
import com.borland.jbuilder.node.*;
import com.borland.primetime.build.*;
import com.borland.primetime.node.*;

public class DeployBuilder
    extends Builder {
  public DeployBuilder() {}

  /**
   * Register open tool
   *
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
    try {
      BuilderManager.registerBuilder(new DeployBuilder());
    }
    catch (Exception e) {
      System.err.println(e.getMessage());
    }
  }

  /**
   * Called by the Build System when make or rebuild occurs
   */
  public void updateBuildProcess(BuildProcess buildProcess,
                                 Node nativeNode) {
    DeployBuildTask task = null;

    task = (DeployBuildTask) buildProcess.getFirstBuildTask(DeployBuildTask.
        SHARED_KEY, DeployBuildTask.class);
    if (task == null) {
      buildProcess.createTask(DeployBuildTask.class, DeployBuildTask.SHARED_KEY);
      task = null;
    }
    if (isMakeable(nativeNode)) {
      String prjPath = buildProcess.getProject().getProjectPath().getFullName();
      try {
        AntProject antProject = new AntProject(prjPath);
        if (antProject.getProperty(
            AntProject.DEPLOY_INVOKE).equalsIgnoreCase("true")) {
          buildProcess.addDependency(DeployBuildTask.SHARED_KEY,
                                     Phase.DEPLOY_PHASE);
        }
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  public void endUpdateBuildProcess(BuildProcess buildProcess) {

  }

  public boolean isMakeable(Node node) {
    if ((node instanceof JBProject) &&
        node.getProperty("enhydra.project", "false").equalsIgnoreCase("true") )
      return true;
    else
      return false;
  }
}
