/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;

// Enhydra imports
import org.enhydra.xml.xmlc.XMLCException;

// JBuilder imports
import com.borland.jbuilder.JBuilderHelp;
import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.help.ZipHelpBook;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserListener;
import com.borland.primetime.ide.NodeViewer;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.InvalidUrlException;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import javax.swing.JOptionPane;
import java.awt.Container;
import java.io.File;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
abstract public class AbstractBasicWizard extends BasicWizard {
    static ResourceBundle         res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");    // nores

    //
    private JBProject             nativeProject = null;

    //
    public static boolean         enableMenu = true;
    public static BrowserListener browserListener =
        new AbstractBasicWizardBrowserListener();

    public JBProject getNativeProject() {
        return nativeProject;
    }

    public void setNativeProject(JBProject p) {
        nativeProject = p;
    }

    public void saveNodes(OtterNode[] nodes) {
        if (nodes != null) {
            for (int i = 0; i < nodes.length; i++) {
                nodes[i].save();
            }
        }
    }

    public abstract String getHelpTopic();
    public void help(WizardPage wizardPage, WizardHost wizardHost) {
        ZipHelpTopic topic = null;
        ZipHelpBook  book = null;

        book = JBuilderHelp.createBook(Constants.KELP);
        topic = new ZipHelpTopic(book, getHelpTopic());
        topic.show(wizardPage.getPageComponent(wizardHost));
    }

    public WizardPage invokeWizard(WizardHost host) {
        setWizardTitle(getName());
        return super.invokeWizard(host);
    }

    /**
     * Method declaration
     *
     *
     * @throws VetoException
     *
     * @see
     */
    protected void finish() throws VetoException {}

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public static String getName() {
        return "Enhydra Wizard";
    }

    public static class AbstractBasicWizardBrowserListener
        implements BrowserListener {
        public void browserActivated(Browser browser) {
            refreshMenu(browser);
        }

        public void browserClosed(Browser browser) {
            refreshMenu(browser);
        }

        public void browserClosing(Browser browser) throws VetoException {
            refreshMenu(browser);
        }

        public void browserDeactivated(Browser browser) {
            refreshMenu(browser);
        }

        public void browserOpened(Browser browser) {
            refreshMenu(browser);
        }

        public void browserProjectActivated(Browser browser,
                                            Project project) {
            refreshMenu(browser);
        }

        public void browserProjectClosed(Browser browser, Project project) {
            refreshMenu(browser);
        }

        public void browserNodeActivated(Browser browser, Node node) {
            refreshMenu(browser);
        }

        public void browserNodeClosed(Browser browser, Node node) {
            refreshMenu(browser);
        }

        public void browserViewerActivated(Browser b, Node n, NodeViewer v) {
            refreshMenu(b);
        }

        public void browserViewerDeactivating(Browser b, Node n, NodeViewer v)
                throws VetoException {
            refreshMenu(b);
        }

        private void refreshMenu(Browser b) {
            Project ap = b.getActiveProject();

            AbstractBasicWizard.enableMenu = (ap != null);
        }

    }

    /**
     * Method declaration
     *
     *
     * @param projectClassPath
     *
     * @return
     *
     * @see
     */
    public boolean verifyAllClassPaths(Container owner) {
        boolean valid = false;

        try {
           KelpInfo.verifyIDEClassPath();
           valid = true;
        } catch (Exception e) {
           JOptionPane.showMessageDialog(owner,
                      e.getMessage(),
                      KelpInfo.getMessageBoxTitle(),
                      JOptionPane.ERROR_MESSAGE);
        }
        if (valid) {
            valid =
                nativeProject.getPaths().putClassOnFullPath(ToolBoxInfo.XMLC_VERSION_CLASS);
            if (!valid) {
                StringBuffer buf = new StringBuffer();

                buf.append(res.getString("EnhydraLibError1"));
                buf.append('\n');
                buf.append(res.getString("EnhydraLibError2"));
                JOptionPane.showMessageDialog(owner, buf.toString(),
                                              Constants.KELP,
                                              JOptionPane.WARNING_MESSAGE);
            }
        }
        return (valid);
    }

}
