package org.enhydra.kelp.ant;


import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import org.apache.crimson.tree.TextNode;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.Method;

/**
 * @author Tweety
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AntXMLUtil {


	//xml document
	public Document document;


	//xml document name
	private String xmlFileName;



    AntXMLUtil() throws Exception {
    	this("");
    }


	AntXMLUtil(String fileName) throws Exception {

        this.xmlFileName = fileName;
        try{
            DOMParser parser = new DOMParser();
            parser.parse(xmlFileName);
            document = parser.getDocument();

        }catch(Exception e){
            throw e;
        }

    }



	/*
	 * @return Node that represents project tag
	 */
	public Node getProjectNode() {
		NodeList list = document.getElementsByTagName("project");
		if(list.getLength() > 0)
			return list.item(0);
		else
			return null;
	}

	/*
	 * @param propertyName name attribute of the property tag
	 * @return Node that represents property tag
	 */
	public Node getProjectProperty(String propertyName) {
		Node project = this.getProjectNode();
		NodeList childs = project.getChildNodes();
		for(int i = 0; i < childs.getLength(); i++) {
			if(childs.item(i).getNodeName().equals("property")) {
				Node nameAttribute = childs.item(i).getAttributes().getNamedItem("name");
				if(nameAttribute != null && nameAttribute.getNodeValue().equals(propertyName))
					return childs.item(i);
			}
		}
		return null;
	}


	/*
	 * @param targetName: name attribute of the searched target
	 * @return Node that represents target with given name
	 */
	public Node getTargetByName(String targetName) {
		NodeList allTargets = document.getElementsByTagName("target");
		for(int i = 0; i < allTargets.getLength(); i++) {
			String nameValue = allTargets.item(i).getAttributes().getNamedItem("name").getNodeValue();

			if(nameValue.equals(targetName))
				return allTargets.item(i);
		}
		return null;
	}


	/*
	 * This method finds child Node in given subtree, with given name
	 * @param subRoot: root node
	 * @param childName: child tag name
	 */
	public Node getChildByTagName(Node subRoot, String tagName) {
		NodeList childs = subRoot.getChildNodes();
			for(int i = 0; i < childs.getLength(); i++) {
				if(childs.item(i).getNodeName().equals(tagName)) {
					return childs.item(i);
				}
			}
		return null;
	}

	/*
	 * This method finds child attribute Node in given subtree, with given name
	 * @param subRoot: root node
	 * @param attributeName: name of attribute
	 */
	public Node getAttributeByName(Node subRoot, String attributeName) {
		return subRoot.getAttributes().getNamedItem(attributeName);
	}

	/*
	 * This method finds patternset Node with given id
	 * @param id: id of searched patternset
	 */
	public Node getPatternsetById(String id) {
		Node projectNode = this.getProjectNode();
		NodeList childs = projectNode.getChildNodes();
		for(int i = 0; i < childs.getLength(); i++) {
			if(childs.item(i).getNodeName().equals("patternset")) {
				if(childs.item(i).getAttributes().getNamedItem("id").getNodeValue().equals(id))
					return childs.item(i);
			}
		}

		return null;
	}

	/*
	 * This method finds filterset Node with given id
	 * @param id: id of searched filterset
	 */
	public Node getFiltersetById(String id) {
		Node projectNode = this.getProjectNode();
		NodeList childs = projectNode.getChildNodes();
		for(int i = 0; i < childs.getLength(); i++) {
			if(childs.item(i).getNodeName().equals("filterset")) {
				if(childs.item(i).getAttributes().getNamedItem("id").getNodeValue().equalsIgnoreCase(id))
					return childs.item(i);
			}
		}

		return null;
	}

	public boolean saveDocument() {

            try{
                FileOutputStream os = new FileOutputStream(xmlFileName);
                OutputFormat of = new OutputFormat(); //xerces apache.xml.serializer
                of.setIndenting(true);
                of.setIndent(4);
                of.setMethod(Method.XML);
                of.setPreserveSpace(true);
                XMLSerializer out = new XMLSerializer(os,of);
                //IOException
                out.serialize(document);
            } catch (Exception e) {
			e.printStackTrace();
                      return false;
            }
            return true;
	}

}
