package org.enhydra.kelp.ant.xmlc;

/**
 * <p>Title: </p>
 * <p>Description: New dialog for Kelp XML Compiler based on Ant tool.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Damir Milovic
 * @version 1.0
 */

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.SwingUtil;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.swing.AddinInnerPanel;
import org.enhydra.kelp.common.swing.FileNodeSelectionPanel;
//import org.enhydra.kelp.common.xmlc.XMLCOutputPanel;
import org.enhydra.kelp.ant.swing.AntOutputPanel;


// JDK
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.awt.*;
import java.beans.*;
import javax.swing.*;

//
public class AntXMLCInnerPanel extends AddinInnerPanel {

    //
    static ResourceBundle          res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private boolean                standalone = false;

    private FileNodeSelectionPanel selectionPanel = null;
    private GridBagLayout          layoutMain = null;
    private AntOutputPanel        outputPanel = null;
    private JTabbedPane            tab = null;
    private WriteListener[]        writeListeners = new WriteListener[0];
    private String                 progressTitle = new String();
    AntXMLCOptionsPanel               optionsPanel = null;

    public static AntXMLCInnerPanel XMLCfindAncestor(Component comp) {
        Component      found = null;
        AntXMLCInnerPanel ancestor = null;

        found = AddinInnerPanel.findAncestor(comp);
        if (found == null) {}
        else if (found instanceof AntXMLCInnerPanel) {
            ancestor = (AntXMLCInnerPanel) found;
        }
        return ancestor;
    }

    public AntXMLCInnerPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // overwrite org.enhydra.kelp.common.swing.InnerPanel
    public void clearAll() {
        super.clearAll();
        layoutMain = null;
        writeListeners = new WriteListener[0];
    }
    //7.2.2003
    public FileNodeSelectionPanel getSelectionPanel() {
        return selectionPanel;
    }
    protected void setSelectText(String[] s) {
        getSelectionPanel().setSelectText(s);
    }
    protected String[] getSelectText() {
        return getSelectionPanel().getSelectText();
    }
    /**
     * Get source files that can be selected for compilation.
     */
    protected OtterFileNode[] getNodes() {
        return selectionPanel.getNodes();
    }
    /**
     * Set the source files that can be selected for compilation
     */
    protected void setNodes(OtterFileNode[] n) {
        selectionPanel.setNodes(n);
    }


    public AntXMLCOptionsPanel getOptionsPanel() {
        return optionsPanel;
    }

    public void selectOutputTab() {
        selectTab(outputPanel);
    }

    public void selectOptionTab() {
        selectTab(optionsPanel);
    }

    //
    // PROTECTED
    //

    public synchronized WriteListener[] getWriteListeners() {
        return writeListeners;
    }

    public synchronized void addWriteListener(WriteListener l) {
        ArrayList list = null;
        list = new ArrayList(Arrays.asList(writeListeners));
        if (!list.contains(l)) {
            list.add(l);
            list.trimToSize();
            writeListeners = new WriteListener[list.size()];
            writeListeners = (WriteListener[]) list.toArray(writeListeners);
        }
        list.clear();
    }

    public synchronized void removeWriteListener(WriteListener l) {
        ArrayList list = null;
        list = new ArrayList(Arrays.asList(writeListeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            writeListeners = new WriteListener[list.size()];
            writeListeners = (WriteListener[]) list.toArray(writeListeners);
        }
        list.clear();
    }


    // override AddinInnerPanel
    public void read(OtterNode node) {
        super.read(node);
        outputPanel.setProject(getProject());
        optionsPanel.setProject(getProject());
    }

    // override AddinInnerPanel
    public void write(OtterNode node) throws DataValidationException {
        super.write(node);
        if (getProject() == null) {
            System.err.println("AntXMLCInnerPanel.write(OtterProject p) : no project set");
        } else {
            outputPanel.setProject(getProject());
        }
    }

    // override AddinInnerPanel
    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[1];

        comps[0] = tab;
        return comps;
    }

    //
    // PRIVATE
    //
    // override AddinInnerPanel
    public void save() {
        super.save();
//       FIXME project.save()
    }
//DACHA{
    //protected void clearOutput() {
    public void clearOutput() {
//}DACHA
        outputPanel.clearOutput();
    }

    //
    // PRIVATE
    //

    private void selectTab(final JPanel goPanel) {
        SelectTab runSwing = null;

        runSwing = new SelectTab(tab, goPanel);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runSwing.run();
            } else {
                SwingUtilities.invokeAndWait(runSwing);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {

            //
        }
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
        tab.add(res.getString("Selections"), selectionPanel);
        tab.add(res.getString("Options"), optionsPanel);
        tab.add(res.getString("Output"), outputPanel);
        addWriteListener(outputPanel);
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());

        optionsPanel = (AntXMLCOptionsPanel) Beans.instantiate(getClass().getClassLoader(),
                                                AntXMLCOptionsPanel.class.getName());

        selectionPanel =
            (FileNodeSelectionPanel) Beans.instantiate(getClass().getClassLoader(),
                FileNodeSelectionPanel.class.getName());

        outputPanel =
            (AntOutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                                AntOutputPanel.class.getName());
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());

        this.setLayout(layoutMain);
        this.add(tab,
                 new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 5, 5));
    }

    private class SelectTab implements Runnable {
        private JTabbedPane tp = null;
        private JPanel      go = null;

        protected SelectTab(JTabbedPane t, JPanel g) {
            tp = t;
            go = g;
        }

        synchronized public void run() {
            tp.setSelectedComponent(go);
            try {
                SwingUtilities.windowForComponent(tp).repaint();
            } catch (NullPointerException e) {

                // ignore for windows not yet open.
            }
        }

    }

}
