/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ProgressBuilder;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.event.WriteListener;

// JDK
import java.io.PrintWriter;
import java.io.File;
import java.util.StringTokenizer;

/**
 *
 *
 * @author Paul Mahar
 */
abstract public class AbstractNodeBuilder extends AbstractEchoBuilder {
    private OtterFileNode[]  nodes = null;

    /**
     * Create a CompileThread with listeners ready to
     * recieve output from XMLC.
     *
     *
     * @param listeners
     * A listener implements an onWrite method that is called
     * whenever XMLC writes output. The writes are captured
     * for display in a wizard.
     */
    public AbstractNodeBuilder(WriteListener[] listeners) {
        super();
        for (int i = 0; i < listeners.length; i++) {
            addWriteListener(listeners[i]);
        }
    }

    public AbstractNodeBuilder(WriteListener listener, boolean buffer) {
        super(listener, buffer);
    }


    /**
     * Determine what files were created and write out the
     * build results to any registered listeners.
     */
    abstract public void printResults();

    /**
     * Get the file nodes that are to be built.
     */
    public OtterFileNode[] getNodes() {
        return nodes;
    }

    /**
     * Set the file nodes to compile.
     */
    public void setNodes(OtterFileNode[] n) {
        nodes = n;
    }

    public void printExceptions() {
        StringTokenizer tokenizer = null;
        int             eCount = 0;
        int             totalCount = getNodes().length;

        for (int i = 0; i < totalCount; i++) {
            if (getNodes()[i].isSelected()
                    && getNodes()[i].getException() != null) {
                eCount++;
            }
        }
        if (eCount > 0) {
            getEchoWriter().println(new String());
            getEchoWriter().println("Exception encountered:");
            for (int i = 0; i < totalCount; i++) {
                if (getNodes()[i].getException() != null) {
                    getEchoWriter().print(Constants.TAB4);
                    getEchoWriter().print(getNodes()[i].getFilePath());
                    getEchoWriter().println(':');
                    tokenizer =
                        new StringTokenizer(getNodes()[i].getException().getMessage(),
                                            Constants.NEWLINE);
                    while (tokenizer.hasMoreTokens()) {
                        getEchoWriter().println(tokenizer.nextToken());
                    }
                }
            }
        }
    }

    /**
     * Sleep the thread for a given number of seconds.
     *
     *
     * @param seconds
     * How many seconds to sleep.
     */
    public void sleepInSeconds(double seconds) {
        try {
            Thread.sleep((int) (seconds * 1000));
        } catch (InterruptedException e) {}
    }

}
