/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.codegen;

// ToolBox
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.codegen.ProjectGenerator;
import org.enhydra.tool.codegen.ProjectOptionSet;
import org.enhydra.tool.codegen.CodeGen;
import org.enhydra.tool.codegen.OptionSet;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.wizard.CodeGenWizard;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.importer.ImportPaths;
import org.enhydra.kelp.common.importer.ImportTool;
import org.enhydra.kelp.common.ValidationException;

// JDK
import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

//
abstract public class ProjectGenUtil {

    // strings not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private final static String EXCEPTION_CREATE_GENERATOR =
        "ProjectGenUtil.createGenerator()";                         // nores

    //
    private OtterProject        project = null;
    private CodeGen             codeGen = null;
    private int                 option = JOptionPane.CANCEL_OPTION;

    public static ProjectGenerator createGenerator() throws CodeGenException {
        throw new CodeGenException(ProjectGenUtil.EXCEPTION_CREATE_GENERATOR);
    }

    /**
     * Constructor declaration
     *
     *
     */
    public ProjectGenUtil(ProjectGenerator gen) throws CodeGenException {
        try {
            codeGen = new CodeGen(gen);
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    public ProjectGenUtil(OtterProject proj,
                          ProjectGenerator gen) throws CodeGenException {
        this(gen);
        init(proj);
    }

    public CodeGen getCodeGen() {
        return codeGen;
    }

    public void init(OtterProject proj) throws CodeGenException {
        project = proj;
        try {
            initOptions();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     */
    public CodeGenPanel[] getWizardPanels() throws CodeGenException {
        CodeGenPanel[] panels = null;

        try {
            panels = getGenerator().getWizardPanels();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return panels;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     */
    public File[] generate() throws CodeGenException {
        File[] files = new File[0];

        try {
            for (int i = 0; i < getGenerator().getWizardPanels().length;
                    i++) {
                getGenerator().getWizardPanels()[i].writeOptionSet();
            }
            getGenerator().setEcho(false);
            files = getGenerator().generate();
        } catch (GeneratorException e) {
            e.printStackTrace(System.err);
            throw new CodeGenException(e);
        }
        importFiles(files);
        return files;
    }

    public ProjectGenerator getGenerator() {
        ProjectGenerator generator = null;

        generator = (ProjectGenerator) codeGen.getSelection();
        return generator;
    }

    /**
     * Method declaration
     *
     *
     * @param owner
     *
     * @return
     */
    public File[] invokeWizard(Component owner) throws CodeGenException {
        File[] files = new File[0];

        files = codeGen.invokeWizard(owner);
        option = codeGen.getOption();
        if (option == JOptionPane.CANCEL_OPTION) {

            // done
        } else if (files != null) {
            importFiles(files);
        }
        return files;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public int getOption() {
        return option;
    }

    //
    //

    /**
     * Method declaration
     *
     * @return
     */
    private String getProjectSourcePath() throws CodeGenException {
        String path = new String();

        try {
            path = getGenerator().getProjectSourcePath();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return path;
    }

    protected ImportPaths getImportPaths() throws CodeGenException {
        ImportPaths paths = null;
        String[][]  map = new String[0][2];
        String      pack = new String();

        paths = new ImportPaths();
        try {
            pack = getPackage();
            paths.setRootPath(getGenerator().getDestination());
            paths.setPackage(pack);
            paths.setPackageMap(map);
            paths.setSourcePath(getProjectSourcePath());
            paths.setProject(project);
        } catch (ValidationException e) {
            throw new CodeGenException(e);
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return paths;
    }

    /**
     * Method declaration
     *
     * @param files
     */
    public void importFiles(File[] files) throws CodeGenException {
        ImportTool  tool = null;
        ImportPaths paths = null;

        if (files.length > 0) {
            tool = new ImportTool();
            paths = getImportPaths();
            tool.setPaths(getImportPaths());
            tool.importFiles(files);
            String[] path = new String[1];

            path[0] = paths.getSourcePath();
            project.setSourcePathArray(path);
        }
    }

    /**
     * Method declaration
     *
     *
     * @param p
     */
    protected void initOptions() throws GeneratorException {
        File       file = null;
        PathHandle phDest = null;
        String     projectName = new String();
        String     pack = new String();
        int        index = -1;

        phDest =
            PathHandle.createPathHandle(project.getCodeGenDefaultDestination());
        projectName = project.getProjectRootName();
        pack = projectName.toLowerCase();
        getGenerator().getOptionSet().lookup(ProjectOptionSet.ROOT).setValue(phDest.getPath());
        getGenerator().getOptionSet().lookup(ProjectOptionSet.PROJECT).setValue(projectName);
        getGenerator().getOptionSet().lookup(ProjectOptionSet.PACKAGE).setValue(pack);
        getGenerator().getOptionSet().lookup(ProjectOptionSet.NOCLI).setValue(true);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     */
    private String getPackage() throws GeneratorException {
        OptionSet optionSet;
        String    pack;

        optionSet = getGenerator().getOptionSet();
        pack = optionSet.lookup(ProjectOptionSet.PACKAGE).getValue();
        return pack;
    }

}
