/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// ToolBox imports
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.common.PathHandle;

// Standard imports
import javax.swing.table.DefaultTableModel;

//
public class ArchiveTableModel extends DefaultTableModel {
    //
    public ArchiveTableModel() {
        super();
        addColumn("Built");
        addColumn("Path");
    }

    public void setPlans(JarPlan[] plans) {
        PathHandle ph = null;

        clear();
        for (int i = 0; i < plans.length; i++) {
            addPlan(plans[i]);
        }
    }

    public JarPlan getPlan(int index) {
       JarPlan plan = null;
       if ((index > -1) && (index < getRowCount())) {
         plan = (JarPlan) getValueAt(index, 1);
       }
       return plan;
    }

    public void setPlan(JarPlan plan, int index) {
       if ((index > -1) && (index < getRowCount())) {
         setValueAt(plan, index, 1);
       }

    }

    public JarPlan[] getPlans() {
        JarPlan[] plans = new JarPlan[0];
        int       count = getRowCount();

        plans = new JarPlan[count];
        for (int i = 0; i < count; i++) {
            plans[i] = (JarPlan) getValueAt(i, 1);
        }
        return plans;
    }

    public int addPlan(JarPlan newPlan) {
        int        count = getRowCount();
        int        index = -1;
        JarPlan    plan = null;
        Object[]   row = new Object[0];
        PathHandle ph = null;

        for (int i = 0; i < count; i++) {
            plan = (JarPlan) getValueAt(i, 1);
            if (plan.getArchivePath().equals(newPlan.getArchivePath())) {
                setValueAt(newPlan, i, 1);
                index = i;
                break;
            }
        }
        if (index == -1) {
            ph = PathHandle.createPathHandle(newPlan.getArchivePath());
            row = new Object[2];
            row[0] = new Boolean(ph.isFile());
            row[1] = newPlan;
            addRow(row);
            index = getRowCount() - 1;
        }
        return index;
    }

    public boolean isCellEditable(int row, int col) {
        return false;
    }

    protected void clear() {
        while (getRowCount() > 0) {
            removeRow(0);
        }
    }

}
