/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// ToolBox imports
import org.enhydra.tool.common.ButtonPanel;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import javax.swing.*;

//
public class DeployButtonPanel extends ButtonPanel {
    //
    private GridBagLayout       layoutMain = null;
    private JButton             buttonBack = null;
    private JButton             buttonNext = null;
    private JButton             buttonDeploy = null;
    private JButton             buttonClose = null;
    private JButton             buttonHelp = null;
    private JButton             buttonAbout = null;
    private LocalButtonListener buttonListener = null;
    private JPanel              panelFiller;

    public static final String[] getOptions() {
      final String[] options = {ButtonPanel.COMMAND_BACK,
         ButtonPanel.COMMAND_NEXT, ButtonPanel.COMMAND_DEPLOY,
         ButtonPanel.COMMAND_CLOSE, ButtonPanel.COMMAND_HELP,
         ButtonPanel.COMMAND_ABOUT};
      return options;
    }

    public DeployButtonPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // override ButtonPanel
    public void removeHelp() {
        remove(buttonHelp);
        invalidate();
        doLayout();
    }

    //
    //
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonBack.addActionListener(buttonListener);
        buttonNext.addActionListener(buttonListener);
        buttonDeploy.addActionListener(buttonListener);
        buttonClose.addActionListener(buttonListener);
        buttonAbout.addActionListener(buttonListener);
        buttonHelp.addActionListener(buttonListener);

        buttonBack.setText(ButtonPanel.COMMAND_BACK);
        buttonNext.setText(ButtonPanel.COMMAND_NEXT);
        buttonDeploy.setText(ButtonPanel.COMMAND_DEPLOY);
        buttonClose.setText(ButtonPanel.COMMAND_CLOSE);
        buttonAbout.setText(ButtonPanel.COMMAND_ABOUT);
        buttonHelp.setText(ButtonPanel.COMMAND_HELP);
        //
        buttonBack.setActionCommand(buttonBack.getText());
        buttonNext.setActionCommand(buttonNext.getText());
        buttonDeploy.setActionCommand(buttonDeploy.getText());
        buttonClose.setActionCommand(buttonClose.getText());
        buttonAbout.setActionCommand(buttonAbout.getText());
        buttonHelp.setActionCommand(buttonHelp.getText());
        //
        buttonClose.requestFocus();
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonBack = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonNext = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonDeploy =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonClose =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonAbout = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonHelp = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        buttonBack.setText("BACK");
        buttonNext.setText("NEXT");
        buttonDeploy.setText("DEPLOY");
        buttonClose.setText("CLOSE");
        buttonAbout.setText("ABOUT");
        buttonHelp.setText("HELP");
        this.setLayout(layoutMain);
        this.add(buttonBack, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 20, 5, 2), 0, 0));
        this.add(buttonNext, new GridBagConstraints(2, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 2, 5, 2), 0, 0));
        this.add(buttonDeploy, new GridBagConstraints(3, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 2, 5, 2), 0, 0));
        this.add(buttonClose, new GridBagConstraints(4, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 2, 5, 2), 0, 0));
        this.add(buttonAbout, new GridBagConstraints(6, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 2, 5, 10), 0, 0));
        this.add(buttonHelp, new GridBagConstraints(5, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 2, 5, 2), 0, 0));
        this.add(panelFiller, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 50, 0));
    }

    private class LocalButtonListener implements ActionListener {

        /**
         * ActionListener implementation.
         *
         * @param event
         * Event that triggered this listener.
         *
         */
        public void actionPerformed(ActionEvent e) {
            Object      source = null;
            ActionEvent event = null;

            source = e.getSource();
            if (source == buttonBack) {
                event = new ActionEvent(buttonBack, ButtonPanel.BACK,
                                        buttonBack.getActionCommand());
            } else if (source == buttonNext) {
                event = new ActionEvent(buttonNext, ButtonPanel.NEXT,
                                        buttonNext.getActionCommand());
            } else if (source == buttonDeploy) {
                event = new ActionEvent(buttonDeploy,
                                        ButtonPanel.DEPLOY,
                                        buttonDeploy.getActionCommand());
            } else if (source == buttonClose) {
                event = new ActionEvent(buttonClose,
                                        ButtonPanel.CLOSE,
                                        buttonClose.getActionCommand());
            } else if (source == buttonHelp) {
                event = new ActionEvent(buttonHelp, ButtonPanel.HELP,
                                        buttonHelp.getActionCommand());
            } else if (source == buttonAbout) {
                event = new ActionEvent(buttonAbout, ButtonPanel.ABOUT,
                                        buttonAbout.getActionCommand());
            }
            if (event != null) {
                notifyActionListeners(event);
            }
        }

    }
}
