/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.dods;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterProject;

// JDK
import java.io.File;

//
public class RunParameters {
    private OtterProject project = null;
    private boolean is3 = false;

    //
    public RunParameters(OtterProject p) {
        project = p;
        is3 = ToolBoxInfo.isEnhydra3();

    }

    public String getAppParameters() {
        StringBuffer buf = new StringBuffer();
        buf.append('\"');
        buf.append(project.getDeployBootstrapPath());
        buf.append('\"');
        return buf.toString();
   }

   public String getClasspathArg()
   {
       if (is3) {
           return "";
       }
       else {
            StringBuffer buf = new StringBuffer();
            String       eRoot = new String();

            eRoot = ToolBoxInfo.getEnhydraRoot().replace('\\', '/');
            buf.setLength(0);
            buf.append("-classpath ");
            buf.append('\"');
            buf.append(eRoot);
            buf.append("/lib/Boot.jar");
            buf.append(File.pathSeparatorChar);
            buf.append(eRoot);
            buf.append("/lib/EAAL.jar");
            buf.append('\"');
            return buf.toString();
       }
   }

   public String getBootArg() {
       if (is3) {
           return "";
       } else {
            StringBuffer bootPath = new StringBuffer();

            bootPath.append("-Dorg.enhydra.boot.properties=");    // nores
            bootPath.append('\"');
            bootPath.append(project.getDeployRootPath());
            bootPath.append(File.separator);
            bootPath.append("boot.properties");
            bootPath.append('\"');
            return bootPath.toString().replace('\\', '/');
       }
   }

/*   public String getPolicyArg() {
       if (is3) {
           return "";
       } else {
            StringBuffer policyPath = new StringBuffer();
            StringBuffer buf = new StringBuffer();

            policyPath.append(PathUtil.getDeployConfPath(project));
            policyPath.append(File.separator);
            policyPath.append(Constants.FILE_JAVA_POLICY);

            File policy = new File(policyPath.toString().replace('\\', '/'));
            if (!policy.exists()) {
                policy = RunGenerator.createPolicyFile(policyPath.toString());
            }

            buf.append("-Djava.security.policy=");           // nores
            buf.append('\"');
            buf.append(policyPath);
            buf.append('\"');
            return buf.toString();
       }
   } */

   public String getTCcpArg() {
       if (is3) {
           return "";
       } else {
           StringBuffer buf = new StringBuffer();
           String eRoot = ToolBoxInfo.getEnhydraRoot().replace('\\', '/');

            buf.append("-Dtc_path_add=");
            buf.append('\"');
            buf.append(eRoot);
            buf.append("/lib/WebService.jar");
            buf.append(File.pathSeparator);
            buf.append(eRoot);
            buf.append("/lib/SharedApiService.jar");
            buf.append('\"');
            return buf.toString();
       }
   }
    public String getVMParameters() {
        StringBuffer buf = new StringBuffer();

        if (is3) {

            // done
        } else {
            buf.setLength(0);
            buf.append(' ');
            buf.append(getClasspathArg());
            buf.append(' ');
            buf.append(getTCcpArg());
            buf.append(' ');
//            buf.append(getPolicyArg());
            buf.append(' ');
            buf.append(getBootArg());
        }
        return buf.toString();
    }

}
