/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.importer;

// Toolbox imports
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ExtensionFilter;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.FileFilter;
import java.io.File;
//
public class ResourceFilter extends ExtensionFilter implements FileFilter {
    private OtterProject project = null;
    private boolean      resourcePath = false;
    private String[]     contentTypes = null;

    /**
     * Creates an extension file filter without an extensions set.
     */
    public ResourceFilter() {
        setDirectoryValid(false);
    }

    public boolean isResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(boolean b) {
        resourcePath = b;
    }

    /**
     */
    public boolean accept(File f) {
        boolean    acceptFile = super.accept(f);
        PathHandle thisPath = null;

        if (acceptFile && (getProject() == null)) {
            acceptFile = false;
        }
        if (acceptFile && isResourcePath()) {
            int index = -1;
            StringBuffer buf = new StringBuffer();
            buf.append('/');
            buf.append(Constants.DIR_RESOURCES);
            buf.append('/');

            thisPath = PathHandle.createPathHandle(f);
            index = thisPath.getPath().toLowerCase().indexOf(buf.toString());
            if (index == -1) {
                acceptFile = false;
            }
        }
        if (acceptFile) {
            PathHandle skipPath = null;
            StringBuffer buf = new StringBuffer();

            thisPath = PathHandle.createPathHandle(f);
            buf.append(getProject().getRootPath());
            buf.append(File.separator);
            buf.append(getProject().getProjectRootName());
            buf.append('.');
            buf.append(Constants.TYPE_HTML);
            skipPath = PathHandle.createPathHandle(buf.toString());
            if (thisPath.equals(skipPath)) {
                acceptFile = false;
            }
        }
        return acceptFile;
    }

    public OtterProject getProject() {
        return project;
    }

    public void setProject(OtterProject p) {
        project = p;
        if (contentTypes == null) {
            contentTypes = project.getContentTypes();
            for (int i = 0; i < contentTypes.length; i++) {
                addExtension(contentTypes[i]);
            }
        }
    }

}
