/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.properties;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.DataValidationException;

// AddinCore
import org.enhydra.kelp.common.event.ExtensionChangeEvent;
import org.enhydra.kelp.common.event.ExtensionChangeListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.swing.AddinInnerPanel;
import org.enhydra.kelp.common.swing.ExtensionPanel;
import org.enhydra.kelp.common.xmlc.XMLCOptionPanel;
import org.enhydra.kelp.common.xmlc.XMLCInnerPanel;
import org.enhydra.kelp.common.xmlc.MapDialog;

// JDK
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import java.util.ResourceBundle;

//
public class XMLCProjectPropertyPanel extends AddinInnerPanel
    implements ExtensionChangeListener {

    // strings OK to resource
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private final String        mapAll = res.getString("All");
    private final String        mapSelected = res.getString("Selected");
    private final String        mapNone = res.getString("None");

    //
    private boolean             active = false;
    private GridBagLayout       layoutCompile;
    private TitledBorder        borderMappings;
    private JPanel              panelMappings;
    private GridBagLayout       layoutMap;
    private JButton             buttonMap;
    private JPanel              panelTrace;
    private GridBagLayout       layoutTrace;
    private JCheckBox           checkPrintParseInfo;
    private JCheckBox           checkPrintAccessorInfo;
    private JCheckBox           checkPrintDocumentInfo;
    private JCheckBox           checkPrintDOM;
    private JCheckBox           checkVerbose;
    private JComboBox           comboMap;
    private JLabel              labelMap;
    private ExtensionPanel      extensionPanel;
    private XMLCOptionPanel     optionPanel;
    private JPanel              panelMake;
    private JCheckBox           checkBuild;
    private GridBagLayout       layoutMake;
    private JTabbedPane         tabPane;
    private JPanel              panelCompile;
    private BorderLayout        layoutMain;
    private LocalButtonListener buttonListener = null;

    /**
     * Constructor declaration
     *
     */
    public XMLCProjectPropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ExtensionChangeListener
    public void onChange(ExtensionChangeEvent event) {
        XMLCInnerPanel innerPanel = null;

        innerPanel = XMLCInnerPanel.XMLCfindAncestor(this);
        if (innerPanel == null) {

            // nothing to refresh;
        } else {
            getProject().setDocTypes(event.getExtensions());
            innerPanel.getSelectionPanel().setNodes(getProject().getAllDocuments());
        }
    }

    // override AddinInnerPanel
    public void read(OtterNode node) {
        super.read(node);
        OtterProject project = null;

        project = getProject();
        optionPanel.setNode(project);
        optionPanel.readProperties();
        if (comboMap.getItemCount() > 0) {
            comboMap.removeAllItems();
        }
        if (project.isOpenBuild()) {

            // Primetime
            checkBuild.setEnabled(true);
            checkBuild.setSelected(project.isXMLCBuild());
            comboMap.addItem(mapAll);
            comboMap.addItem(mapSelected);
            comboMap.addItem(mapNone);
            int vScope = project.getVerifiedMapScope();

            switch (vScope) {
            case OtterProject.MAP_SCOPE_NONE:
                comboMap.setSelectedItem(mapNone);
                break;
            case OtterProject.MAP_SCOPE_SELECTED:
                comboMap.setSelectedItem(mapSelected);
                break;
            default:
                project.setMapScope(OtterProject.MAP_SCOPE_ALL);
                comboMap.setSelectedItem(mapAll);
                break;
            }
        } else {

            // Win32 and JDeveloper
            checkBuild.setEnabled(false);
            checkBuild.setSelected(false);
            comboMap.addItem(mapAll);
            comboMap.addItem(mapNone);
            if (project.getMapScope() == OtterProject.MAP_SCOPE_NONE) {
                comboMap.setSelectedItem(mapNone);
            } else {
                comboMap.setSelectedItem(mapAll);
                project.setMapScope(OtterProject.MAP_SCOPE_ALL);
            }
        }
        checkPrintDOM.setSelected(project.isPrintDOM());
        checkPrintParseInfo.setSelected(project.isPrintParseInfo());
        checkPrintAccessorInfo.setSelected(project.isPrintAccessorInfo());
        checkPrintDocumentInfo.setSelected(project.isPrintDocumentInfo());
        checkVerbose.setSelected(project.isVerbose());
        extensionPanel.setExtensions(project.getDocTypes());
        extensionPanel.setDefaults(ToolBoxInfo.getSupportedDocTypes());
        active = true;
    }

    // override AddinInnerPanel
    public void write(OtterNode node) throws DataValidationException {
        super.write(node);
        String       item = null;
        OtterProject project = null;

        project = getProject();
        item = comboMap.getSelectedItem().toString().trim();
        optionPanel.writeProperties();
        if (project != null && active) {
            if (item.equalsIgnoreCase(mapAll)) {
                project.setMapScope(OtterProject.MAP_SCOPE_ALL);
            } else if (item.equalsIgnoreCase(mapSelected)) {
                project.setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            } else if (item.equalsIgnoreCase(mapNone)) {
                project.setMapScope(OtterProject.MAP_SCOPE_NONE);
            }
        }
        if (project.isOpenBuild()) {
            // Primetime
            project.setXMLCBuild(checkBuild.isSelected());
        } else {

            // Win32 and JDeveloper
            project.setXMLCBuild(false);
        }
        project.setPrintDOM(checkPrintDOM.isSelected());
        project.setPrintParseInfo(checkPrintParseInfo.isSelected());
        project.setPrintAccessorInfo(checkPrintAccessorInfo.isSelected());
        project.setPrintDocumentInfo(checkPrintDocumentInfo.isSelected());
        project.setVerbose(checkVerbose.isSelected());
        project.setDocTypes(extensionPanel.getExtensions());
    }

    // override AddinInnerPanel
    public Component[] getFirstFocusComponents() {
        Component[] comps = new Component[14];

        comps[0] = panelMappings;
        comps[1] = buttonMap;
        comps[2] = checkPrintParseInfo;
        comps[3] = checkPrintAccessorInfo;
        comps[4] = checkPrintDocumentInfo;
        comps[5] = checkPrintDOM;
        comps[6] = checkVerbose;
        comps[7] = comboMap;
        comps[8] = extensionPanel;
        comps[9] = optionPanel;
        comps[10] = panelMake;
        comps[11] = checkBuild;
        comps[12] = tabPane;
        comps[13] = panelCompile;
        return comps;
    }

    /**
     * Method declaration
     *
     */
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonMap.addActionListener(buttonListener);
        extensionPanel.addListener(this);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        layoutCompile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        extensionPanel =
            (ExtensionPanel) Beans.instantiate(getClass().getClassLoader(),
                                               ExtensionPanel.class.getName());
        optionPanel =
            (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOptionPanel.class.getName());
        borderMappings = new TitledBorder(new String());
        panelMappings =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutMap =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonMap = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        panelTrace = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        layoutTrace =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkPrintParseInfo =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkPrintAccessorInfo =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkPrintDocumentInfo =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkPrintDOM =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkVerbose =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        comboMap = (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                                 JComboBox.class.getName());
        labelMap = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                              JLabel.class.getName());
        panelMake = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        checkBuild =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMake =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tabPane =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        panelCompile = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        borderMappings.setTitle(res.getString("borderMappings_Title"));
        panelMappings.setBorder(borderMappings);
        panelTrace.setLayout(layoutTrace);
        panelMappings.setLayout(layoutMap);
        buttonMap.setText(res.getString("buttonMap_Text"));
        checkPrintParseInfo.setText(res.getString("checkPrintParseInfo_Text"));
        checkPrintDocumentInfo.setText(res.getString("checkPrintDocumentInfo_Text"));
        checkPrintDOM.setText(res.getString("checkPrintDOM_Text"));
        checkVerbose.setText(res.getString("checkVerbose_Text"));
        checkPrintAccessorInfo.setText(res.getString("checkPrintAccessorInfo_Text"));
        labelMap.setHorizontalAlignment(SwingConstants.RIGHT);
        labelMap.setText(res.getString("labelMap_Text"));
        checkBuild.setEnabled(false);
        checkBuild.setText(res.getString("checkBuild_Text"));
        panelMake.setLayout(layoutMake);
        panelMake.add(checkBuild,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 2, 2, 2), 0, 0));
        panelMappings.add(buttonMap,
                          new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(2, 2, 1, 2), 0,
                                                 0));
        panelMappings.add(comboMap,
                          new GridBagConstraints(1, 0, 1, 1, 0.2, 0.2,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.HORIZONTAL,
                                                 new Insets(2, 2, 1, 2), 0,
                                                 0));
        panelMappings.add(labelMap,
                          new GridBagConstraints(0, 0, 1, 1, 0.2, 0.2,
                                                 GridBagConstraints.EAST,
                                                 GridBagConstraints.HORIZONTAL,
                                                 new Insets(2, 2, 1, 2), 0,
                                                 0));
        panelCompile.setLayout(layoutCompile);
        panelCompile.add(panelMappings,
                         new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 2, 2, 0), 5,
                                                5));
        panelCompile.add(optionPanel,
                         new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(1, 2, 2, 2), 0,
                                                0));
        panelTrace.add(checkPrintDocumentInfo,
                       new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(0, 0, 0, 0), 0, 0));
        panelTrace.add(checkPrintParseInfo,
                       new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(2, 2, 2, 2), 5, 0));
        panelTrace.add(checkPrintDOM,
                       new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(2, 2, 2, 2), 5, 0));
        panelTrace.add(checkVerbose,
                       new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.NONE,
                                              new Insets(2, 2, 2, 2), 5, 0));
        panelTrace.add(checkPrintAccessorInfo,
                       new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(2, 2, 2, 2), 5, 0));
        tabPane.add(panelCompile, res.getString("Compile"));
        tabPane.add(extensionPanel, res.getString("XMLC_Types"));
        tabPane.add(panelTrace, res.getString("Trace"));
        this.setLayout(layoutMain);
        this.add(tabPane, BorderLayout.CENTER);
        this.add(panelMake, BorderLayout.NORTH);
    }

    /**
     * Method declaration
     *
     */
    private void editProjectMap() {
        MapDialog dialog =
            new MapDialog(((JDialog) this.getTopLevelAncestor()),
                          res.getString("Edit_Project_Map"), true);

        dialog.setProject(getProject());
        dialog.show();
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonMap) {
                editProjectMap();
            }
        }

    }
}
