/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.swing;

// Toolbox
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.DataValidationException;

// AddinCore
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;

// JDK
import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.beans.*;
import java.lang.ref.WeakReference;
import javax.swing.*;

public class AddinInnerPanel extends InnerPanel {

    private WeakReference nodeRef = null;

    public AddinInnerPanel() {
        super();
    }

    // override InnerPanel
    public void clearAll() {
        super.clearAll();
        if (nodeRef != null) {
            nodeRef.clear();
        }
        nodeRef = null;
    }

    // override InnerPanel
    public void save() {
        try {
            write(getNode());
        } catch (DataValidationException e) {
            System.err.println(e.getValidationMessage());
        }
    }

    public void read(OtterNode node) {
        nodeRef = new WeakReference(node);
    }

    public void write(OtterNode node) throws DataValidationException {
        nodeRef = new WeakReference(node);
    }

    protected OtterNode getNode() {
        OtterNode node = null;

        if (nodeRef != null) {
            node = (OtterNode) nodeRef.get();
        }
        return node;
    }

    protected OtterProject getProject() {
        OtterNode    node = null;
        OtterProject project = null;

        node = getNode();
        if (node == null) {
            project = null;
        } else if (node instanceof OtterProject) {
            project = (OtterProject) node;
        } else {
            project = node.getProject();
        }
        return project;
    }

}
