/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.File;
import java.beans.*;
public class ConfPanel extends JPanel {
    private LocalActionListener actionListener = null;
    private GridBagLayout       layoutMain;
    private JLabel              labelHome;
    private JTextField          textHome;
    private JButton             buttonHome;
    private JLabel              labelConfFile;
    private JTextField          textConfFile;
    private JButton             buttonConfFile;
    private JLabel              labelConfDir;
    private JTextField          textConfDir;
    private JButton             buttonConfDir;
    private JLabel              labelPolicy;
    private JTextField          textPolicy;
    private JButton             buttonPolicy;
    private JLabel              labelBrowser;
    private JTextField          textBrowser;
    private JButton             buttonBrowser;
    private JPanel              panelFiller;
    private BorderLayout        layoutFiller;

    public ConfPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    protected void firstFocus() {
        textHome.requestFocus();
    }

    protected void initPaths() {
        textHome.setText(getApp().getHome());
        textHome.setToolTipText(textHome.getText());
        textConfFile.setText(getApp().getConfFile());
        textConfFile.setToolTipText(textConfFile.getText());
        textConfDir.setText(getApp().getConfDir());
        textConfDir.setToolTipText(textConfDir.getText());
        textPolicy.setText(getApp().getPolicy());
        textPolicy.setToolTipText(textPolicy.getText());
        textBrowser.setText(getApp().getBrowser());
        textBrowser.setToolTipText(textBrowser.getText());
    }

    private LaunchApp getApp() {
        return LaunchApp.app;
    }

    private void selectHome() {
        File selection = null;

        selection = new File(textHome.getText());
        selection =
            SwingUtil.getDirectoryChoice(this, selection,
                                         "Select Enhydra Home Directory");
        if (selection != null) {
            textHome.setText(selection.getAbsolutePath());
            textHome.setToolTipText(textConfDir.getText());
            getApp().setHome(textHome.getText());
        }
    }

    private void selectConfFile() {
        File            selection = null;
        ExtensionFilter filter = new ExtensionFilter();

        filter.setDescriptionTitle("Configuration");
        filter.addExtension("conf");
        selection = new File(textConfFile.getText());
        selection = SwingUtil.getFileChoice(this, selection, filter,
                                            "Select Configuration File");
        if (selection != null) {
            textConfFile.setText(selection.getAbsolutePath());
            textConfFile.setToolTipText(textConfFile.getText());
            getApp().setConfFile(textConfFile.getText());
        }
    }

    private void selectConfDir() {
        File selection = null;

        selection = new File(textConfDir.getText());
        selection =
            SwingUtil.getDirectoryChoice(this, selection,
                                         "Select Configuration Directory");
        if (selection != null) {
            textConfDir.setText(selection.getAbsolutePath());
            textConfDir.setToolTipText(textConfDir.getText());
            getApp().setConfDir(textConfDir.getText());
        }
    }

    private void selectPolicyFile() {
        File            selection = null;
        ExtensionFilter filter = new ExtensionFilter();

        filter.setDescriptionTitle("Policy");
        filter.addExtension("policy");
        selection = new File(textPolicy.getText());
        selection = SwingUtil.getFileChoice(this, selection, filter,
                                            "Select Policy File");
        if (selection != null) {
            textPolicy.setText(selection.getAbsolutePath());
            textPolicy.setToolTipText(textPolicy.getText());
            getApp().setPolicy(textPolicy.getText());
        }
    }

    protected void selectBrowser() {
        File            selection = null;
        ExtensionFilter filter = new ExtensionFilter();

        filter.setDescriptionTitle("Browser");
        if (System.getProperty("os.name").toLowerCase().indexOf("win") > -1) {
            filter.addExtension("exe");
        }
        selection = new File(textBrowser.getText());
        selection = SwingUtil.getFileChoice(this, selection, filter,
                                            "Select Web Browser");
        if (selection != null) {
            textBrowser.setText(selection.getAbsolutePath());
            textBrowser.setToolTipText(textBrowser.getText());
            getApp().setBrowser(textBrowser.getText());
        }
    }

    private void pmInit() {
        textHome.setPreferredSize(new Dimension(100,
                                                textHome.getPreferredSize().getSize().height));
        textHome.setText(new String());
        textConfFile.setPreferredSize(new Dimension(100,
                                                    textConfFile.getPreferredSize().getSize().height));
        textConfFile.setText(new String());
        textConfDir.setPreferredSize(new Dimension(100,
                                                   textConfDir.getPreferredSize().getSize().height));
        textConfDir.setText(new String());
        textPolicy.setPreferredSize(new Dimension(100,
                                                  textPolicy.getPreferredSize().getSize().height));
        textPolicy.setText(new String());
        actionListener = new LocalActionListener();
        buttonHome.addActionListener(actionListener);
        buttonConfFile.addActionListener(actionListener);
        buttonConfDir.addActionListener(actionListener);
        buttonPolicy.addActionListener(actionListener);
        buttonBrowser.addActionListener(actionListener);
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelHome = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        labelConfFile =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelConfDir = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        labelPolicy = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                 JLabel.class.getName());
        labelBrowser = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textHome = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        textConfFile =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textConfDir =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textPolicy =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textBrowser =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonHome = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonConfFile =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonConfDir =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonPolicy =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonBrowser =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        layoutFiller =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        textHome.setEditable(false);
        textConfFile.setEditable(false);
        textConfDir.setEditable(false);
        textPolicy.setEditable(false);
        textBrowser.setEditable(false);
        buttonHome.setText("...");
        buttonConfFile.setText("...");
        buttonConfDir.setText("...");
        buttonPolicy.setText("...");
        buttonBrowser.setText("...");
        labelHome.setText("Enhydra home directory:");
        labelHome.setDisplayedMnemonic('E');
        labelHome.setLabelFor(textHome);
        labelConfFile.setText("Bootstrap configuration file:");
        labelConfFile.setDisplayedMnemonic('S');
        labelConfFile.setLabelFor(textConfFile);
        labelConfDir.setText("Configuration directory:");
        labelConfDir.setDisplayedMnemonic('C');
        labelConfDir.setLabelFor(textConfDir);
        labelPolicy.setText("Java policy file:");
        labelPolicy.setDisplayedMnemonic('P');
        labelPolicy.setLabelFor(textPolicy);
        labelBrowser.setText("Browser:");
        labelBrowser.setDisplayedMnemonic('B');
        labelBrowser.setLabelFor(textBrowser);
        panelFiller.setLayout(layoutFiller);
        this.setLayout(layoutMain);
        this.add(labelHome,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 2, 5), 0, 0));
        this.add(textHome,
                 new GridBagConstraints(1, 0, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 2, 5), 100, 0));
        this.add(buttonHome,
                 new GridBagConstraints(2, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 2, 5), 0, 0));
        this.add(labelConfFile,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(textConfFile,
                 new GridBagConstraints(1, 1, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 100, 0));
        this.add(buttonConfFile,
                 new GridBagConstraints(2, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(labelConfDir,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(textConfDir,
                 new GridBagConstraints(1, 2, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 100, 0));
        this.add(buttonConfDir,
                 new GridBagConstraints(2, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(labelPolicy,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(textPolicy,
                 new GridBagConstraints(1, 3, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 100, 0));
        this.add(buttonPolicy,
                 new GridBagConstraints(2, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(labelBrowser,
                 new GridBagConstraints(0, 4, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(textBrowser,
                 new GridBagConstraints(1, 4, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 100, 0));
        this.add(buttonBrowser,
                 new GridBagConstraints(2, 4, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(panelFiller,
                 new GridBagConstraints(0, 5, 3, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 20, 20));
    }

    class LocalActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonHome) {
                selectHome();
            } else if (source == buttonConfFile) {
                selectConfFile();
            } else if (source == buttonConfDir) {
                selectConfDir();
            } else if (source == buttonPolicy) {
                selectPolicyFile();
            } else if (source == buttonBrowser) {
                selectBrowser();
            }
        }

    }
}
