/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch;
import java.io.File;
import java.util.Vector;
import javax.swing.filechooser.FileFilter;

/**
 * An implementation of FileFilter that filters out all files
 * except for those matching a set of given extensions.
 */
public class ExtensionFilter extends FileFilter
    implements java.io.FileFilter {
    private Vector  ext = new Vector();
    private String  descTitle = new String();
    private boolean directoryValid = true;

    /**
     * Creates an extension file filter without an extensions set.
     */
    public ExtensionFilter() {}

    /**
     * Return true if this file should be shown in a file chooser.
     */
    public boolean accept(File f) {
        boolean acceptFile = false;
        int     count = ext.size();
        String  fileExt = getExtension(f);

        if (f == null) {
            acceptFile = true;
        } else if (f.isDirectory()) {
            acceptFile = isDirectoryValid();
        } else if (fileExt != null) {
            for (int i = 0; i < count; i++) {
                String current = (String) ext.elementAt(i);

                if (fileExt.equalsIgnoreCase(current)) {
                    acceptFile = true;
                }
            }
        }
        return acceptFile;
    }

    /**
     * Return the file extension.
     */
    private String getExtension(File f) {
        String ex = null;

        if (f != null) {
            String filename = f.getName();
            int    i = filename.lastIndexOf('.');

            if (i > 0 && i < filename.length() - 1) {
                ex = filename.substring(i + 1).toLowerCase().trim();
            }
        }
        return ex;
    }

    /**
     * Adds an an extension to the list of acceptable file
     * types.
     */
    public void addExtension(String e) {
        ext.addElement(e.toLowerCase().trim());
    }

    /**
     * Returns the human readable description of this filter.
     */
    public String getDescription() {
        StringBuffer desc = new StringBuffer();

        desc.append(getDescriptionTitle());
        desc.append(" (");
        if (ext == null) {
            desc.append("*.*");
        } else {
            desc.append("*.");
            int count = ext.size();

            for (int i = 0; i < count; i++) {
                desc.append((String) ext.elementAt(i));
                if (i < (count - 1)) {
                    desc.append(", ");
                }
            }
        }
        desc.append(")");
        return desc.toString();
    }

    /**
     * Sets the portion of the description that does not
     * include extensions.
     */
    public void setDescriptionTitle(String d) {
        this.descTitle = d;
    }

    /**
     * Gets the portion of the description that does not
     * include extensions.
     */
    public String getDescriptionTitle() {
        return descTitle;
    }

    public void setDirectoryValid(boolean valid) {
        directoryValid = valid;
    }

    public boolean isDirectoryValid() {
        return directoryValid;
    }

}
