/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch;

// Kernel imports
import org.enhydra.multiServer.kernel.Kernel;

// Standard mports
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;

//
public class LaunchFrame extends JFrame {
    private BorderLayout         layoutRoot;
    private LaunchPanel          launchPanel;
    private GridBagLayout        layoutMain;
    private JMenuBar             menuBar;
    private JMenu                menuFile;
    private JMenuItem            menuStart;
    private JMenuItem            menuStop;
    private JMenuItem            menuGotoAdmin;
    private JMenuItem            menuExit;
    private JMenu                menuView;
    private JRadioButtonMenuItem menuMessages;
    private JRadioButtonMenuItem menuSettings;
    private JMenu                menuHelp;
    private JMenuItem            menuAdminHelp;
    private JMenuItem            menuAbout;

    public LaunchFrame() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void initPaths() {
        launchPanel.initPaths();
    }

    protected LaunchApp getApp() {
        return LaunchApp.app;
    }

    protected void clearAll() {
        setVisible(false);
        removeAll();
        dispose();
    }

    protected void refreshStatus(String status, String message,
                                 boolean starting) {
        launchPanel.refreshStatus(status, message, starting);
        if (starting) {
            menuStart.setEnabled(false);
            menuStop.setEnabled(true);
            menuGotoAdmin.setEnabled(false);
        } else if (Kernel.getKernel() == null) {
            menuStart.setEnabled(true);
            menuStop.setEnabled(false);
            menuGotoAdmin.setEnabled(false);
        } else if (getApp().isServerRunning()) {
            menuStart.setEnabled(false);
            menuStop.setEnabled(true);
            menuGotoAdmin.setEnabled(true);
        } else {
            menuStart.setEnabled(true);
            menuStop.setEnabled(true);
            menuGotoAdmin.setEnabled(false);
        }
    }

    private void pmInit() {
        ButtonGroup group = new ButtonGroup();

        group.add(menuMessages);
        group.add(menuSettings);

        //
    }

    public void show() {
        pack();
        setSize((int) (getSize().width * 0.8),
                (int) (getSize().height * 0.8));
        setLocation(SwingUtil.getCenteringPoint(getSize()));
        super.show();
    }

    protected void processWindowEvent(java.awt.event.WindowEvent e) {
        if (e.getID() == java.awt.event.WindowEvent.WINDOW_CLOSING) {
            exit();
        }
    }

    private void jbInit() throws Exception {
        launchPanel =
            (LaunchPanel) Beans.instantiate(getClass().getClassLoader(),
                                            LaunchPanel.class.getName());
        menuBar = (JMenuBar) Beans.instantiate(getClass().getClassLoader(),
                                               JMenuBar.class.getName());
        menuFile = (JMenu) Beans.instantiate(getClass().getClassLoader(),
                                             JMenu.class.getName());
        menuStart = (JMenuItem) Beans.instantiate(getClass().getClassLoader(),
                                                  JMenuItem.class.getName());
        menuStop = (JMenuItem) Beans.instantiate(getClass().getClassLoader(),
                                                 JMenuItem.class.getName());
        menuGotoAdmin =
            (JMenuItem) Beans.instantiate(getClass().getClassLoader(),
                                          JMenuItem.class.getName());
        menuView = (JMenu) Beans.instantiate(getClass().getClassLoader(),
                                             JMenu.class.getName());
        menuMessages =
            (JRadioButtonMenuItem) Beans.instantiate(getClass().getClassLoader(),
                JRadioButtonMenuItem.class.getName());
        menuSettings =
            (JRadioButtonMenuItem) Beans.instantiate(getClass().getClassLoader(),
                JRadioButtonMenuItem.class.getName());
        menuExit = (JMenuItem) Beans.instantiate(getClass().getClassLoader(),
                                                 JMenuItem.class.getName());
        menuHelp = (JMenu) Beans.instantiate(getClass().getClassLoader(),
                                             JMenu.class.getName());
        menuAdminHelp =
            (JMenuItem) Beans.instantiate(getClass().getClassLoader(),
                                          JMenuItem.class.getName());
        menuAbout = (JMenuItem) Beans.instantiate(getClass().getClassLoader(),
                                                  JMenuItem.class.getName());
        layoutRoot =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        menuBar.add(menuFile);
        menuBar.add(menuView);
        menuBar.add(menuHelp);
        menuFile.setMnemonic('F');
        menuFile.setText("File");
        menuFile.add(menuStart);
        menuFile.add(menuStop);
        menuFile.addSeparator();
        menuFile.add(menuGotoAdmin);
        menuFile.addSeparator();
        menuFile.add(menuExit);
        menuStart.setMnemonic('S');
        menuStart.setText("Start Server");
        menuStart.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                launchPanel.startServer();
            }

        });
        menuStop.setEnabled(false);
        menuStop.setMnemonic('d');
        menuStop.setText("Shutdown Server");
        menuStop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                launchPanel.stopServer();
            }

        });
        menuGotoAdmin.setEnabled(false);
        menuGotoAdmin.setMnemonic('A');
        menuGotoAdmin.setText("Go to Admin Servlet");
        menuGotoAdmin.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                launchPanel.gotoAdmin();
            }

        });
        menuExit.setMnemonic('X');
        menuExit.setText("Exit");
        menuExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exit();
            }

        });
        menuHelp.setMnemonic('H');
        menuHelp.setText("Help");
        menuAbout.setMnemonic('A');
        menuAbout.setText("About...");
        menuAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                showAboutBox();
            }

        });
        menuAdminHelp.setMnemonic('L');
        menuAdminHelp.setText("Admin Login");
        menuAdminHelp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                adminHelp();
            }

        });
        menuView.setMnemonic('V');
        menuView.setText("View");
        menuView.add(menuMessages);
        menuView.add(menuSettings);
        menuView.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}

            public void menuDeselected(MenuEvent e) {}

            public void menuSelected(MenuEvent e) {
                if (launchPanel.isMessageTabSelected()) {
                    menuMessages.setSelected(true);
                    menuSettings.setSelected(false);
                } else {
                    menuMessages.setSelected(false);
                    menuSettings.setSelected(true);
                }
            }

        });
        menuMessages.setSelected(true);
        menuMessages.setText("Messages");
        menuMessages.setAccelerator(javax.swing.KeyStroke.getKeyStroke(77,
                java.awt.event.KeyEvent.CTRL_MASK, false));
        menuMessages.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    launchPanel.viewMessages();
                }
            }

        });
        menuSettings.setText("Settings");
        menuSettings.setAccelerator(javax.swing.KeyStroke.getKeyStroke(83,
                java.awt.event.KeyEvent.CTRL_MASK, false));
        menuSettings.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    launchPanel.viewSettings();
                }
            }

        });
        menuHelp.add(menuAdminHelp);
        menuHelp.add(menuAbout);
        this.setTitle("Enhydra Launch Utility");
        this.setJMenuBar(menuBar);
        this.getContentPane().setLayout(layoutRoot);
        this.getContentPane().add(launchPanel, BorderLayout.CENTER);
    }

    private void exit() {
        getApp().exit();
    }

    private void showAboutBox() {
        AboutBox about = new AboutBox(this);

        about.setTitleText(getTitle());
        about.setTextAreaText("Enhydra 4.0");
        about.setSmallText("Copyright (c) 2000, Lutris Technologies, All Rights Reserved.");
        about.show();
    }

    private void adminHelp() {
        JOptionPane.showMessageDialog(this,
                                      "Default login for the Admin Servlet\n\n"
                                      + "    User Name: admin\n"
                                      + "    Password: enhydra\n\n",
                                      "Admin Login Help",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

}
