
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.wizard.TBWizardDeck;

// JDK
import java.awt.Component;
import java.lang.ref.WeakReference;

//
public class ArchiveDeck extends TBWizardDeck {
    private ArchiveType   selection = null;
    private boolean       selectionHidden = false;
    private WeakReference wizardRef = null;

    public ArchiveDeck(ArchiveWizard wizard) {
        super();
        wizardRef = new WeakReference(wizard);
    }

    protected void setSelectionPageHidden(boolean b) {
        selectionHidden = b && (selection != null);
        if (selectionHidden && (getPageIndex() == 0)) {
            next();
        }
    }

    protected boolean isSelectionPageHidden() {
        return selectionHidden;
    }

    /**
     * Method declaration
     *
     *
     * @param selection
     */
    protected void setSelection(ArchiveType select) throws ArchiveException {
        selection = select;
        if (selection == null) {

            // do no more
        } else {
            while (getPages().length > 1) {
                removeWizardPage(getPages()[1]);
            }
            for (int i = 0; i < selection.getWizardPanels().length; i++) {
                ArchivePage  newPage = new ArchivePage();
                ArchivePanel panel = selection.getWizardPanels()[i];

                panel.setWizard(getWizard());
                newPage.add(panel);
                newPage.setPageTitle(panel.getPageTitle());
                newPage.setInstructions(panel.getInstructions());
                addWizardPage(newPage);
            }
        }
    }

    protected ArchiveType getSelection() {
        return selection;
    }

    protected boolean isSelectionPageShowing() {
        return getPageIndex() == 0;
    }

    // override TBWizardDeck
    protected boolean isFirstPageShowing() {
        boolean atFirst = super.isFirstPageShowing();

        if (isSelectionPageHidden()) {
            atFirst = (getPageIndex() == 1);
        }
        return (atFirst);
    }

    // Override TBWizardDeck
    protected boolean isLastPageShowing() {
        boolean atLast = super.isLastPageShowing();

        if (!isSelectionPageHidden()) {
            if (isSelectionPageShowing()) {
                atLast = false;
            }
        }
        return (atLast);
    }

    protected void readPlan() throws ArchiveException {
        ArchivePage page = null;

        for (int i = 0; i < getPages().length; i++) {
            page = (ArchivePage) getPages()[i];
            page.readPlan(selection.getPlan());
        }
    }

    protected void validatePlan() throws ArchiveException {
        ArchivePage page = null;

        // validate all pages before writing any
        try {
            for (int i = 0; i < getPages().length; i++) {
                page = (ArchivePage) getPages()[i];
                page.validatePage();
            }
        } catch (ToolException e) {
            throw new ArchiveException(e, e.getMessage());
        }
    }

    protected void writePlan() throws ArchiveException {
        ArchivePage page = null;

        for (int i = 0; i < getPages().length; i++) {
            page = (ArchivePage) getPages()[i];
            page.writePlan(selection.getPlan());
        }
    }

    private ArchiveWizard getWizard() {
        ArchiveWizard wizard = null;

        if (wizardRef != null) {
            wizard = (ArchiveWizard) wizardRef.get();
        }
        return wizard;
    }

}
