
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

package org.enhydra.tool.archive.wizard;

//
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ClassFilter;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.WarPlan;
import org.enhydra.tool.common.PathHandle;

//
import java.awt.*;
import java.beans.Beans;
import java.io.File;
import java.lang.ref.WeakReference;
import javax.swing.*;

//
public class ClassesPanel extends ArchivePanel {
    private BorderLayout          layoutMain = null;
    private TwoColumnFileSelector selectorPanel = null;

    public ClassesPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void validatePlan() throws ArchiveException {
        selectorPanel.validatePlan();
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        selectorPanel.setRoot(plan.getClassRoot());
        selectorPanel.setSelections(plan.getClassFileArray());
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        validatePlan();
        plan.setClassRoot(selectorPanel.getRoot());
        plan.setClassFiles(selectorPanel.getSelections());
    }

    public String getPageTitle() {
        return "Classes";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Select files to include in classes");
        return buf.toString();
    }

    protected void preparePanel() {
        if (selectorPanel.getRootFile() == null) {
            StringBuffer buf = new StringBuffer();
            PathHandle   path = null;

            buf.append(getWorkingRoot());
            buf.append(File.separator);
            buf.append("classes");
            path = PathHandle.createPathHandle(buf.toString());
            if (path.isDirectory()) {
                selectorPanel.setDefaultSelect(true);
            } else {
                selectorPanel.setDefaultSelect(false);
                path = PathHandle.createPathHandle(getWorkingRoot());
            }
            if (path.isDirectory()) {
                selectorPanel.setRoot(path.getPath());
            }
        }
    }

    private void pmInit() {
        selectorPanel.setFilter(new ClassFilter());
    }

    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        selectorPanel =
            (TwoColumnFileSelector) Beans.instantiate(getClass().getClassLoader(),
                TwoColumnFileSelector.class.getName());
        this.setLayout(layoutMain);
        this.add(selectorPanel, BorderLayout.CENTER);
    }

}
