
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.EarPlan;

// JDK
import java.awt.*;
import java.beans.Beans;
import java.io.File;
import java.util.ArrayList;
import javax.swing.*;

//
public class MetaPanel extends ArchivePanel {
    private BorderLayout          layoutMain = null;
    private OneColumnFileSelector selectorPanel = null;

    public MetaPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        selectorPanel.setSelections(plan.getLibFiles());
        Descriptor[] dd = new Descriptor[0];
        String[] paths = new String[0];
        ArrayList list = new ArrayList();

        if (plan instanceof EarPlan) {
            EarPlan ep = (EarPlan) plan;
            dd = ep.getDescriptors();
            for (int i = 0 ; i < dd.length ; i++) {
              list.add(dd[i].getPath());
            }
            list.trimToSize();
            paths = new String[list.size()];
            paths = (String[]) list.toArray(paths);
            selectorPanel.setSelections(paths);
        }
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        if (plan instanceof EarPlan) {
            EarPlan ep = (EarPlan) plan;
            Descriptor[] dd = new Descriptor[0];
            String[] paths = new String[0];
            paths = selectorPanel.getSelections();
            dd = new Descriptor[paths.length];
            for (int i = 0 ; i < dd.length ; i++) {
              dd[i] = new Descriptor(paths[i]);
            }
            ep.setDescriptors(dd);
        }
    }

    public void validatePlan() throws ArchiveException {
    	// always valid
    }

    public String getPageTitle() {
        return "Deployment Descriptors";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Select the deployment descriptors ");
        buf.append("to include in this archive.");
        return buf.toString();
    }

    // override ArchivePanel
    protected void setWizard(ArchiveWizard wizard) {
        super.setWizard(wizard);
        selectorPanel.setWizard(wizard);
    }

    private void pmInit() {
        String[] exts = {
            "xml", "mf"
        };

        selectorPanel.setExtensions(exts);
        selectorPanel.setTitle("meta-inf");
    }

    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        selectorPanel =
            (OneColumnFileSelector) Beans.instantiate(getClass().getClassLoader(),
                OneColumnFileSelector.class.getName());
        this.setLayout(layoutMain);
        this.add(selectorPanel, BorderLayout.CENTER);
    }

}
