
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox imports
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ClassFilter;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

// Standard imports
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Vector;
import java.awt.*;
import javax.swing.*;
import java.beans.*;

/**
 * Class declaration
 *
 * @author Paul Mahar
 */
public class TwoColumnFileSelector extends JPanel {

    // strings not to be resourced
    private final String        SELECT = ">";
    private final String        SELECT_ALL = ">>";
    private final String        REMOVE = "<";
    private final String        REMOVE_ALL = "<<";

    //
    private boolean             defaultSelect = false;
    private JLabel              labelAvailable = null;
    private JLabel              labelSelected = null;
    private JScrollPane         scrollAvailable = null;
    private JScrollPane         scrollSelected = null;
    private JList               listAvailable = null;
    private JList               listSelected = null;
    private JButton             buttonAdd = null;
    private JButton             buttonAddAll = null;
    private JButton             buttonRemove = null;
    private JButton             buttonRemoveAll = null;
    private GridBagLayout       layoutSelect = null;
    private JPanel              panelSelect = null;
    private JPanel              panelOption = null;
    private BorderLayout        layoutMain = null;
    private GridBagLayout       layoutOption = null;
    private JCheckBox           checkShowFull = null;
    private JLabel              labelRoot = null;
    private JTextField          textRoot = null;
    private JButton             buttonRoot = null;
    private LocalButtonListener buttonListener = null;
    private DefaultListModel    modelAvailable = new DefaultListModel();
    private DefaultListModel    modelSelected = new DefaultListModel();
    private ClassFilter         filter = null;

    /**
     * Constructor declaration
     *
     */
    public TwoColumnFileSelector() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void validatePlan() throws ArchiveException {
        String[] selects = new String[0];
        File     file = null;

        if (getRootFile() == null || (!getRootFile().isDirectory())) {
            throw new ArchiveException("Invalid root");
        }
        selects = getSelections();
        for (int i = 0; i < selects.length; i++) {
            file = new File(selects[i]);
            if (!file.isFile()) {
                throw new ArchiveException("Invalid selection: "
                                           + file.getAbsolutePath());
            }
        }
    }

    protected void setFilter(ClassFilter f) {
        filter = f;
    }

    protected ClassFilter getFilter() {
        return filter;
    }

    protected void setDefaultSelect(boolean b) {
        defaultSelect = b;
    }

    protected boolean isDefaultSelect() {
        return defaultSelect;
    }

    private void clearLists() {
        if (getModelSelected().getSize() > 0) {
            getModelSelected().clear();
        }
        if (getModelAvailable().getSize() > 0) {
            getModelAvailable().clear();
        }
    }

    /**
     * Method declaration
     *
     */
    protected void fillFrom(File cursor) throws ListOverflow {
        File[] children = new File[0];

        if (getModelAvailable().size() + getModelSelected().size() > 2500) {
            throw new ListOverflow();
        } else if (cursor == null) {
            children = null;
        } else {
            if (getFilter() == null) {
                children = cursor.listFiles();
            } else {
                getFilter().setRoot(getRoot());
                children = cursor.listFiles(getFilter());
            }
        }
        if (children == null) {

            // done
        } else {
            for (int i = 0; i < children.length; i++) {
                if (children[i].isDirectory()) {
                    fillFrom(children[i]);
                } else if (children[i].isFile()) {
                    LocalListNode listNode = null;

                    listNode =
                        new LocalListNode(children[i].getAbsolutePath());
                    if (isDefaultSelect()) {
                        getModelSelected().addElement(listNode);
                    } else {
                        getModelAvailable().addElement(listNode);
                    }
                }
            }
        }
    }

    // /
    // /

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected DefaultListModel getModelSelected() {
        return modelSelected;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected DefaultListModel getModelAvailable() {
        return modelAvailable;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[] getSelections() {
        String[]    selects = new String[0];
        Enumeration enum = null;
        ArrayList   list = new ArrayList();

        enum = getModelSelected().elements();
        while (enum.hasMoreElements()) {
            LocalListNode next = (LocalListNode) enum.nextElement();

            if (next.getFullname().trim().length() > 0) {
                list.add(next.getFullname());
            }
        }
        list.trimToSize();
        selects = new String[list.size()];
        selects = (String[]) list.toArray(selects);
        return selects;
    }

    public void setSelections(File[] selectFiles) {
        String[] selects = new String[0];

        if (selectFiles != null) {
            selects = new String[selectFiles.length];
            for (int i = 0; i < selects.length; i++) {
                selects[i] = selectFiles[i].getAbsolutePath();
            }
        }
        setSelections(selects);
    }

    public void setSelections(String[] selects) {
        setDefaultSelect(false);
        clearLists();
        fillFromRoot();
        for (int i = 0; i < selects.length; i++) {
            LocalListNode node = null;

            node = new LocalListNode(selects[i]);
            if (!getModelSelected().contains(node)) {
                getModelSelected().addElement(node);
            }
            if (getModelAvailable().contains(node)) {
                getModelAvailable().removeElement(node);
            }
        }
    }

    public void setRoot(String path) {
        PathHandle root = null;

        root = PathHandle.createPathHandle(path);
        if (root.isDirectory()) {
            textRoot.setText(root.getPath());
        } else {
            textRoot.setText(new String());
        }
        textRoot.setToolTipText(textRoot.getText());
        clearLists();
        fillFromRoot();
    }

    public String getRoot() {
        String     root = null;
        PathHandle path = null;

        path = PathHandle.createPathHandle(textRoot.getText());
        if (path.isDirectory()) {
            root = path.getPath();
        }
        return root;
    }

    public File getRootFile() {
        String root = getRoot();
        File   file = null;

        if (root == null) {

            //
        } else {
            file = new File(root);
        }
        return file;
    }

    private void fillFromRoot() {
        try {
            fillFrom(getRootFile());
        } catch (ListOverflow e) {
            JOptionPane.showMessageDialog(this,
                                          "Reached maximum files for root: 2500",
                                          "Archive Wizard Warning",
                                          JOptionPane.WARNING_MESSAGE);
        }
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonRoot.addActionListener(buttonListener);
        buttonAdd.addActionListener(buttonListener);
        buttonAddAll.addActionListener(buttonListener);
        buttonRemove.addActionListener(buttonListener);
        buttonRemoveAll.addActionListener(buttonListener);
        checkShowFull.addActionListener(buttonListener);
        checkShowFull.setSelected(false);
        textRoot.setText(new String());
    }

    private void jbInit() throws Exception {
        layoutSelect =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelSelect = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelOption = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        scrollAvailable =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        scrollSelected =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        labelAvailable =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelSelected =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        listAvailable = (JList) Beans.instantiate(getClass().getClassLoader(),
                                                  JList.class.getName());
        listSelected = (JList) Beans.instantiate(getClass().getClassLoader(),
                                                 JList.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonAddAll =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonRemoveAll =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        layoutOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkShowFull =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        labelRoot = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        textRoot = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonRoot = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        listAvailable.setModel(modelAvailable);
        listSelected.setModel(modelSelected);
        labelAvailable.setText("Available");
        labelSelected.setText("Selected");
        buttonAdd.setMaximumSize(new Dimension(40, 27));
        buttonAdd.setMinimumSize(new Dimension(40, 27));
        buttonAdd.setPreferredSize(new Dimension(40, 27));
        buttonAdd.setToolTipText("Select");
        buttonAdd.setMargin(new Insets(2, 2, 2, 2));
        buttonAdd.setText(SELECT);
        buttonAddAll.setMaximumSize(new Dimension(40, 27));
        buttonAddAll.setMinimumSize(new Dimension(40, 27));
        buttonAddAll.setPreferredSize(new Dimension(40, 27));
        buttonAddAll.setToolTipText("Select All");
        buttonAddAll.setMargin(new Insets(2, 2, 2, 2));
        buttonAddAll.setText(SELECT_ALL);
        buttonRemove.setMaximumSize(new Dimension(40, 27));
        buttonRemove.setMinimumSize(new Dimension(40, 27));
        buttonRemove.setPreferredSize(new Dimension(40, 27));
        buttonRemove.setToolTipText("Deselect");
        buttonRemove.setMargin(new Insets(2, 2, 2, 2));
        buttonRemove.setText(REMOVE);
        buttonRemoveAll.setMaximumSize(new Dimension(40, 27));
        buttonRemoveAll.setMinimumSize(new Dimension(40, 27));
        buttonRemoveAll.setPreferredSize(new Dimension(40, 27));
        buttonRemoveAll.setToolTipText("Deselect All");
        buttonRemoveAll.setMargin(new Insets(2, 2, 2, 2));
        buttonRemoveAll.setText(REMOVE_ALL);
        scrollAvailable.setMaximumSize(new Dimension(300, 300));
        scrollAvailable.setPreferredSize(new Dimension(250, 250));
        scrollSelected.setMaximumSize(new Dimension(300, 300));
        scrollSelected.setPreferredSize(new Dimension(250, 250));
        checkShowFull.setText("Show paths");
        labelRoot.setText("Root:");
        textRoot.setEnabled(false);
        textRoot.setEditable(false);
        textRoot.setText("jTextField1");
        buttonRoot.setMargin(new Insets(2, 6, 2, 6));
        buttonRoot.setText("...");
        scrollAvailable.getViewport().add(listAvailable);
        scrollSelected.getViewport().add(listSelected);
        panelSelect.setLayout(layoutSelect);
        panelSelect.add(labelAvailable,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 2, 5), 0, 0));
        panelSelect.add(scrollAvailable,
                        new GridBagConstraints(0, 1, 1, 4, 0.2, 0.2,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.BOTH,
                                               new Insets(2, 0, 2, 2), 100,
                                               150));
        panelSelect.add(buttonAdd,
                        new GridBagConstraints(1, 1, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(8, 0, 4, 0), 10,
                                               0));
        panelSelect.add(buttonAddAll,
                        new GridBagConstraints(1, 2, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(4, 0, 4, 0), 10,
                                               0));
        panelSelect.add(buttonRemove,
                        new GridBagConstraints(1, 3, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.SOUTH,
                                               GridBagConstraints.NONE,
                                               new Insets(4, 0, 4, 0), 10,
                                               0));
        panelSelect.add(buttonRemoveAll,
                        new GridBagConstraints(1, 4, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(4, 0, 8, 0), 10,
                                               0));
        panelSelect.add(labelSelected,
                        new GridBagConstraints(2, 0, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 2, 5), 0, 0));
        panelSelect.add(scrollSelected,
                        new GridBagConstraints(2, 1, 1, 4, 0.2, 0.2,
                                               GridBagConstraints.EAST,
                                               GridBagConstraints.BOTH,
                                               new Insets(2, 2, 2, 0), 100,
                                               150));
        panelOption.setLayout(layoutOption);
        panelOption.add(labelRoot,
                        new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 15, 5, 0), 0,
                                               0));
        panelOption.add(checkShowFull,
                        new GridBagConstraints(0, 0, 3, 1, 0.5, 0.5,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 15, 5, 5), 0,
                                               0));
        panelOption.add(textRoot,
                        new GridBagConstraints(1, 1, 1, 1, 0.8, 0.8,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 1, 5, 1), 0, 0));
        panelOption.add(buttonRoot,
                        new GridBagConstraints(2, 1, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 1, 5, 1), 0, 0));
        this.setLayout(layoutMain);
        this.setMinimumSize(new Dimension(400, 200));
        this.setPreferredSize(new Dimension(620, 300));
        this.add(panelSelect, BorderLayout.CENTER);
        this.add(panelOption, BorderLayout.SOUTH);
    }

    private void addSelected() {
        Object[] elements = listAvailable.getSelectedValues();

        for (int i = 0; i < elements.length; i++) {
            LocalListNode listNode = (LocalListNode) elements[i];

            modelSelected.addElement(listNode);
            modelAvailable.removeElement(listNode);
        }
    }

    private void refreshLists() {
        int size = modelAvailable.getSize();

        for (int i = 0; i < size; i++) {
            modelAvailable.setElementAt(modelAvailable.getElementAt(i), i);
        }
        size = modelSelected.getSize();
        for (int i = 0; i < size; i++) {
            modelSelected.setElementAt(modelSelected.getElementAt(i), i);
        }
    }

    private void addAllElements() {
        for (int i = 0; i < modelAvailable.size(); i++) {
            modelSelected.addElement(modelAvailable.elementAt(i));
        }
        modelAvailable.removeAllElements();
    }

    private void removeSelected() {
        Object[] elements = listSelected.getSelectedValues();

        for (int i = 0; i < elements.length; i++) {
            LocalListNode listNode = (LocalListNode) elements[i];

            modelAvailable.addElement(listNode);
            modelSelected.removeElement(listNode);
        }
    }

    private void removeAllElements() {
        for (int i = 0; i < modelSelected.size(); i++) {
            modelAvailable.addElement(modelSelected.elementAt(i));
        }
        modelSelected.removeAllElements();
    }

    private void selectRoot() {
        File choice = null;

        choice = SwingUtil.getDirectoryChoice(this, textRoot.getText(),
                                              "Select root directory");
        if (choice == null) {

            // no change
        } else {
            setRoot(choice.getAbsolutePath());
        }
    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonAdd) {
                addSelected();
            } else if (source == buttonAddAll) {
                addAllElements();
            } else if (source == buttonRemove) {
                removeSelected();
            } else if (source == buttonRemoveAll) {
                removeAllElements();
            } else if (source == checkShowFull) {
                refreshLists();
            } else if (source == buttonRoot) {
                selectRoot();
            }
        }

    }

    //
    private class LocalListNode {
        private String full = new String();

        public LocalListNode(String fullName) {
            full = PathHandle.createPathString(fullName);
        }

        public String toString() {
            String s = full;

            if (!checkShowFull.isSelected()) {
                File f = new File(full);

                s = f.getName();
            }
            return s;
        }

        public String getFullname() {
            return full;
        }

        public boolean equals(Object in) {
            boolean eq = false;

            if (in instanceof LocalListNode) {
                LocalListNode comp = (LocalListNode) in;

                eq = comp.getFullname().equals(getFullname());
            }
            return eq;
        }

    }
    private class ListOverflow extends Exception {}
}
