
package org.enhydra.tool.archive.xml;

//
import org.enhydra.tool.archive.ArchiveException;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.OutputFormat;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

//
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.BufferedWriter;
import java.io.BufferedReader;
import java.io.StringWriter;
import java.io.StringReader;

//
abstract public class AbstractDescriptorHandler implements DescriptorHandler {

    //
    private Document     doc = null;
    private String       source = new String();
    private OutputStream outStream = null;
    private StringWriter writer = null;

    //
    public void setOutStream(OutputStream s) {
        outStream = s;
    }

    public OutputStream getOutputStream() {
        return outStream;
    }

    public void setWriter(StringWriter w) {
        writer = w;
    }

    public StringWriter getWriter() {
        return writer;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String s) {
        source = s;
    }

    public InputStream getInputStream() {
        ByteArrayInputStream stream = null;
        byte[]               bytes = new byte[0];

        if (getWriter() != null) {
            getWriter().flush();
            bytes = getWriter().toString().getBytes();
            stream = new ByteArrayInputStream(bytes);
        }
        return stream;
    }

    protected Document getDocument() throws ArchiveException {
        DOMParser       parser = null;
        InputSource     inputSource = null;
        DocTypeFilterReader reader = null;

        try {
            reader = new DocTypeFilterReader(
                 new BufferedReader(
                   new FileReader(getSource())));
        } catch (FileNotFoundException e) {
            throw new ArchiveException(e, "Unable to prep: " + getSource());
        }

        inputSource = new InputSource(reader);
        inputSource.setPublicId(null);
        inputSource.setSystemId(null);
        parser = new DOMParser();

        //
        try {
            parser.parse(inputSource);
        } catch (IOException e) {
            e.printStackTrace();
            throw new ArchiveException(e, "Unable to parse: " + getSource());
        } catch (SAXException e) {
            e.printStackTrace();
            throw new ArchiveException(e, "Unable to parse: " + getSource());
        }
        return parser.getDocument();
    }


    public void prep() throws ArchiveException {
        OutputFormat    format = null;
        XMLSerializer   serializer = null;

        format = new OutputFormat();
        serializer = new XMLSerializer();

        doc = getDocument();
        prepElements();

        //
        format.setDoctype(getPublicID(), getSystemID());
        format.setLineSeparator("\r\n");
        format.setLineWidth(8192);
        format.setIndent(2);
        format.setIndenting(true);
        format.setPreserveSpace(false);

        //
        if (getWriter() != null) {
            serializer.setOutputCharStream(getWriter());
        } else if (getOutputStream() != null) {
            serializer.setOutputByteStream(getOutputStream());
        } else {
            initWriter();
            serializer.setOutputCharStream(getWriter());
        }
        serializer.setOutputFormat(format);
        try {
            serializer.serialize(doc.getDocumentElement());
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       "Unable to serialize prep'd "
                                       + getSource());
        }
    }

    //
    //
    abstract protected void prepElements();
    abstract protected String getPublicID();
    abstract protected String getSystemID();

    //
    //
    protected Document getDoc() {
        return doc;
    }

    protected Element lookup(Element parent, String name) {
        NodeList list = null;
        Node     cursor = null;
        Element  element = null;
        int      len = 0;

        list = parent.getChildNodes();
        len = list.getLength();
        for (int i = 0; i < len; i++) {
            cursor = list.item(i);
            if (cursor.getNodeName().equals(name)
                    && cursor instanceof Element) {
                element = (Element) cursor;
                break;
            }
        }
        return element;
    }

    //
    //
    private void initWriter() {
        setWriter(new StringWriter());
    }

}
