
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance witarh the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the Licenseget.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.boot;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;

// JDK
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import java.net.URLClassLoader;


//
public class StartServer extends Frame {
    String[] args = new String[0];
    Button   button = new Button();
    Process  process = null;

    public static void main(String[] args) {
        StartServer starter = new StartServer();

        starter.args = args;
        starter.setSize(200, 100);
        starter.setLocation(new Point(200, 200));
        starter.show();
        starter.setLocation(new Point(200, 200));    // reset for Linux AWT
        starter.createProcess();
    }

    public StartServer() {
        button.setLabel("Shutdown Server");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                process.destroy();
                System.exit(0);
            }

        });
        add(button);
        enableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);
        setTitle("Application Server");
    }

    protected void processWindowEvent(java.awt.event.WindowEvent e) {
        if (e.getID() == java.awt.event.WindowEvent.WINDOW_CLOSING) {
            process.destroy();
            System.exit(0);
        }
    }

    private String createCommandLine() {


        StringBuffer buf = new StringBuffer();
        File         java = new File(System.getProperty("java.home"));

        buf.append(java.getParentFile());
        buf.append(File.separator + "bin" + File.separator + "java");
        if (isWindows()) {
            buf.append("w");
        }
        buf.append(' ');
        buf.append("-classpath");
        buf.append(' ');
        if (isWindows()) {
           buf.append('"');
        }
        if (ToolBoxInfo.isEnhydra3()) {
            buf.append(getCurrentClassPath());
        } else {
            buf.append(getBootClassPath());
        }
        if (isWindows()) {
           buf.append('"');
        }
        buf.append(' ');
        buf.append(getVMProperties());
        buf.append(' ');
        buf.append(ToolBoxInfo.getEnhydraMainClass());
        for (int i = 0; i < args.length; i++) {
            buf.append(' ');
            if (isWindows()) {
               buf.append('"');
            }
            buf.append(args[i].replace('\\','/'));
            if (isWindows()) {
               buf.append('"');
            }
        }
        return buf.toString();
    }

    private void createProcess() {
        String  commandLine = createCommandLine();
        boolean running = true;

        try {
            System.out.println(commandLine);
            process = Runtime.getRuntime().exec(commandLine);
            BufferedReader errorReader =
                new BufferedReader(new InputStreamReader(process.getErrorStream()));
            BufferedReader inputReader =
                new BufferedReader(new InputStreamReader(process.getInputStream()));

            while (running) {
                try {
                    int exit = process.exitValue();

                    System.out.println("Server shutdown: " + exit);
                    running = false;
                } catch (Exception e) {
                    print(errorReader);
                    print(inputReader);
                }
            }
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }

    private void print(BufferedReader reader) throws IOException {
        String       printLine = null;
        StringBuffer buf = new StringBuffer();
        int          count = 0;

        while (reader.ready()) {
            buf.append((char) reader.read());
            count++;
            if (count == 1000) {
                break;
            }
        }
        System.out.print(buf.toString());
        try {
            Thread.sleep(250);
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
        }
    }

    private String getBootClassPath() {
        StringBuffer buf = new StringBuffer();
        String       eRoot = ToolBoxInfo.getEnhydraRoot();

        if (isWindows()) {
          buf.append('"');
        }
        buf.append(File.pathSeparatorChar);
        buf.append(eRoot);
        buf.append("/lib/Boot.jar");
        buf.append(File.pathSeparatorChar);
        buf.append(eRoot);
        buf.append("/lib/EAAL.jar");
        if (isWindows()) {
          buf.append('"');
        }
        return buf.toString();
    }

    private String getCurrentClassPath() {
        StringBuffer   buf = new StringBuffer();
        ClassLoader    loader = ClassLoader.getSystemClassLoader();
        URLClassLoader urlLoader = (URLClassLoader) loader;
        String         path = new String();

        buf.append(File.pathSeparatorChar);
        for (int i = 0; i < urlLoader.getURLs().length; i++) {
            path = urlLoader.getURLs()[i].getFile().replace('/',
                    File.separatorChar);
            if (path.charAt(0) == '\\') {
                path = path.substring(1);
                if (path.endsWith(File.separator)) {
                    path = path.substring(0, path.length() - 1);
                }
            }
            buf.append(path);
            if ((i + 1) < urlLoader.getURLs().length) {
                buf.append(File.pathSeparatorChar);
            }
        }
        return buf.toString();
    }

    private String getVMProperties() {

        Properties   props = System.getProperties();
        Enumeration  names = System.getProperties().propertyNames();
        StringBuffer buf = new StringBuffer();
        ArrayList    list = new ArrayList();

        list.add("tc_path_add");
        list.add("java.security.policy");
        list.add("org.enhydra.boot.properties");
        list.trimToSize();
        while (names.hasMoreElements()) {
            String name = names.nextElement().toString();
            String value = props.getProperty(name);

            if (list.contains(name)) {
                buf.append("-D");
                buf.append(name);
                buf.append('=');
                if (isWindows()) {
                   buf.append('"');
                }
                buf.append(value.replace('\\','/'));
                if (isWindows()) {
                   buf.append('"');
                }
                buf.append(' ');
                if (name.equals("org.enhydra.boot.properties")) {
                    buf.append(getUserDir(value));
                }
            }
        }
        list.clear();
        return buf.toString();
    }

    private String getUserDir(String bootProp) {
        File         boot = new File(bootProp);
        StringBuffer buf = new StringBuffer();

        System.setProperty("user.dir", boot.getParent());
        buf.append("-D");
        buf.append("user.dir");
        buf.append('=');
        if (isWindows()) {
          buf.append('"');
        }
        buf.append(boot.getParent().replace('\\','/'));
        buf.append('/');
        if (isWindows()) {
          buf.append('"');
        }
        buf.append(' ');
        return buf.toString();
    }

    private boolean isWindows() {
      return (File.separatorChar == '\\');
    }

}
